@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -77,11 +71,11 @@ void QDeclarativeVideo::_q_error(int errorCode, const QString &errorString)
     \brief The Video element allows you to add videos to a scene.
     \inherits Item
 
-    This element is part of the \bold{Qt.multimedia 1.0} module.
+    This element is part of the \bold{QtMultimediaKit 1.1} module.
 
     \qml
     import Qt 4.7
-    import Qt.multimedia 1.0
+    import QtMultimediaKit 1.1
 
     Video {
         id: video
@@ -123,9 +117,6 @@ QDeclarativeVideo::QDeclarativeVideo(QDeclarativeItem *parent)
     , m_graphicsItem(0)
 
 {
-    m_graphicsItem = new QGraphicsVideoItem(this);
-    connect(m_graphicsItem, SIGNAL(nativeSizeChanged(QSizeF)),
-            this, SLOT(_q_nativeSizeChanged(QSizeF)));
 }
 
 QDeclarativeVideo::~QDeclarativeVideo()
@@ -212,36 +203,6 @@ QDeclarativeVideo::Status QDeclarativeVideo::status() const
     return Status(m_status);
 }
 
-/*!
-    \qmlsignal Video::onLoaded()
-
-    This handler is called when the media source has been loaded.
-*/
-
-/*!
-    \qmlsignal Video::onBuffering()
-
-    This handler is called when the media starts buffering.
-*/
-
-/*!
-    \qmlsignal Video::onStalled()
-
-    This handler is called when playback has stalled while the media buffers.
-*/
-
-/*!
-    \qmlsignal Video::onBuffered()
-
-    This handler is called when the media has finished buffering.
-*/
-
-/*!
-    \qmlsignal Video::onEndOfMedia()
-
-    This handler is called when playback stops because end of the media has been reached.
-*/
-
 /*!
     \qmlproperty int Video::duration
 
@@ -276,7 +237,7 @@ QDeclarativeVideo::Status QDeclarativeVideo::status() const
 
 bool QDeclarativeVideo::hasAudio() const
 {
-    return m_playerControl == 0 ? false : m_playerControl->isAudioAvailable();
+    return !m_complete ? false : m_playerControl->isAudioAvailable();
 }
 
 /*!
@@ -287,7 +248,7 @@ bool QDeclarativeVideo::hasAudio() const
 
 bool QDeclarativeVideo::hasVideo() const
 {
-    return m_playerControl == 0 ? false : m_playerControl->isVideoAvailable();
+    return !m_complete ? false : m_playerControl->isVideoAvailable();
 }
 
 /*!
@@ -379,7 +340,7 @@ void QDeclarativeVideo::setFillMode(FillMode mode)
 
 void QDeclarativeVideo::play()
 {
-    if (m_playerControl == 0)
+    if (!m_complete)
         return;
 
     setPaused(false);
@@ -396,7 +357,7 @@ void QDeclarativeVideo::play()
 
 void QDeclarativeVideo::pause()
 {
-    if (m_playerControl == 0)
+    if (!m_complete)
         return;
 
     setPaused(true);
@@ -413,7 +374,7 @@ void QDeclarativeVideo::pause()
 
 void QDeclarativeVideo::stop()
 {
-    if (m_playerControl == 0)
+    if (!m_complete)
         return;
 
     setPlaying(false);
@@ -431,8 +392,12 @@ void QDeclarativeVideo::geometryChanged(const QRectF &newGeometry, const QRectF
     QDeclarativeItem::geometryChanged(newGeometry, oldGeometry);
 }
 
-void QDeclarativeVideo::componentComplete()
+void QDeclarativeVideo::classBegin()
 {
+    m_graphicsItem = new QGraphicsVideoItem(this);
+    connect(m_graphicsItem, SIGNAL(nativeSizeChanged(QSizeF)),
+            this, SLOT(_q_nativeSizeChanged(QSizeF)));
+
     setObject(this);
 
     if (m_mediaService) {
@@ -445,6 +410,11 @@ void QDeclarativeVideo::componentComplete()
     }
 }
 
+void QDeclarativeVideo::componentComplete()
+{
+    QDeclarativeMediaBase::componentComplete();
+}
+
 QT_END_NAMESPACE
 
 // ***************************************
@@ -452,7 +422,7 @@ QT_END_NAMESPACE
 // ***************************************
 
 /*!
-    \qmlproperty variant Video::title
+    \qmlproperty variant Video::metaData.title
 
     This property holds the tile of the media.
 
@@ -460,7 +430,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::subTitle
+    \qmlproperty variant Video::metaData.subTitle
 
     This property holds the sub-title of the media.
 
@@ -468,7 +438,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::author
+    \qmlproperty variant Video::metaData.author
 
     This property holds the author of the media.
 
@@ -476,7 +446,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::comment
+    \qmlproperty variant Video::metaData.comment
 
     This property holds a user comment about the media.
 
@@ -484,7 +454,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::description
+    \qmlproperty variant Video::metaData.description
 
     This property holds a description of the media.
 
@@ -492,7 +462,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::category
+    \qmlproperty variant Video::metaData.category
 
     This property holds the category of the media
 
@@ -500,7 +470,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::genre
+    \qmlproperty variant Video::metaData.genre
 
     This property holds the genre of the media.
 
@@ -508,7 +478,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::year
+    \qmlproperty variant Video::metaData.year
 
     This property holds the year of release of the media.
 
@@ -516,7 +486,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::date
+    \qmlproperty variant Video::metaData.date
 
     This property holds the date of the media.
 
@@ -524,7 +494,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::userRating
+    \qmlproperty variant Video::metaData.userRating
 
     This property holds a user rating of the media in the range of 0 to 100.
 
@@ -532,7 +502,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::keywords
+    \qmlproperty variant Video::metaData.keywords
 
     This property holds a list of keywords describing the media.
 
@@ -540,7 +510,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::language
+    \qmlproperty variant Video::metaData.language
 
     This property holds the language of the media, as an ISO 639-2 code.
 
@@ -548,7 +518,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::publisher
+    \qmlproperty variant Video::metaData.publisher
 
     This property holds the publisher of the media.
 
@@ -556,7 +526,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::copyright
+    \qmlproperty variant Video::metaData.copyright
 
     This property holds the media's copyright notice.
 
@@ -564,7 +534,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::parentalRating
+    \qmlproperty variant Video::metaData.parentalRating
 
     This property holds the parental rating of the media.
 
@@ -572,7 +542,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::ratingOrganisation
+    \qmlproperty variant Video::metaData.ratingOrganisation
 
     This property holds the name of the rating organisation responsible for the
     parental rating of the media.
@@ -581,7 +551,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::size
+    \qmlproperty variant Video::metaData.size
 
     This property property holds the size of the media in bytes.
 
@@ -589,7 +559,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::mediaType
+    \qmlproperty variant Video::metaData.mediaType
 
     This property holds the type of the media.
 
@@ -597,7 +567,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::audioBitRate
+    \qmlproperty variant Video::metaData.audioBitRate
 
     This property holds the bit rate of the media's audio stream ni bits per
     second.
@@ -606,7 +576,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::audioCodec
+    \qmlproperty variant Video::metaData.audioCodec
 
     This property holds the encoding of the media audio stream.
 
@@ -614,7 +584,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::averageLevel
+    \qmlproperty variant Video::metaData.averageLevel
 
     This property holds the average volume level of the media.
 
@@ -622,7 +592,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::channelCount
+    \qmlproperty variant Video::metaData.channelCount
 
     This property holds the number of channels in the media's audio stream.
 
@@ -630,7 +600,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::peakValue
+    \qmlproperty variant Video::metaData.peakValue
 
     This property holds the peak volume of media's audio stream.
 
@@ -638,7 +608,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::sampleRate
+    \qmlproperty variant Video::metaData.sampleRate
 
     This property holds the sample rate of the media's audio stream in hertz.
 
@@ -646,7 +616,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::albumTitle
+    \qmlproperty variant Video::metaData.albumTitle
 
     This property holds the title of the album the media belongs to.
 
@@ -654,7 +624,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::albumArtist
+    \qmlproperty variant Video::metaData.albumArtist
 
     This property holds the name of the principal artist of the album the media
     belongs to.
@@ -663,7 +633,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::contributingArtist
+    \qmlproperty variant Video::metaData.contributingArtist
 
     This property holds the names of artists contributing to the media.
 
@@ -671,7 +641,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::composer
+    \qmlproperty variant Video::metaData.composer
 
     This property holds the composer of the media.
 
@@ -679,7 +649,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::conductor
+    \qmlproperty variant Video::metaData.conductor
 
     This property holds the conductor of the media.
 
@@ -687,7 +657,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::lyrics
+    \qmlproperty variant Video::metaData.lyrics
 
     This property holds the lyrics to the media.
 
@@ -695,7 +665,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::mood
+    \qmlproperty variant Video::metaData.mood
 
     This property holds the mood of the media.
 
@@ -703,7 +673,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::trackNumber
+    \qmlproperty variant Video::metaData.trackNumber
 
     This property holds the track number of the media.
 
@@ -711,7 +681,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::trackCount
+    \qmlproperty variant Video::metaData.trackCount
 
     This property holds the number of track on the album containing the media.
 
@@ -719,7 +689,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::coverArtUrlSmall
+    \qmlproperty variant Video::metaData.coverArtUrlSmall
 
     This property holds the URL of a small cover art image.
 
@@ -727,7 +697,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::coverArtUrlLarge
+    \qmlproperty variant Video::metaData.coverArtUrlLarge
 
     This property holds the URL of a large cover art image.
 
@@ -735,7 +705,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::resolution
+    \qmlproperty variant Video::metaData.resolution
 
     This property holds the dimension of an image or video.
 
@@ -743,7 +713,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::pixelAspectRatio
+    \qmlproperty variant Video::metaData.pixelAspectRatio
 
     This property holds the pixel aspect ratio of an image or video.
 
@@ -751,7 +721,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::videoFrameRate
+    \qmlproperty variant Video::metaData.videoFrameRate
 
     This property holds the frame rate of the media's video stream.
 
@@ -759,7 +729,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::videoBitRate
+    \qmlproperty variant Video::metaData.videoBitRate
 
     This property holds the bit rate of the media's video stream in bits per
     second.
@@ -768,7 +738,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::videoCodec
+    \qmlproperty variant Video::metaData.videoCodec
 
     This property holds the encoding of the media's video stream.
 
@@ -776,7 +746,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::posterUrl
+    \qmlproperty variant Video::metaData.posterUrl
 
     This property holds the URL of a poster image.
 
@@ -784,7 +754,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::chapterNumber
+    \qmlproperty variant Video::metaData.chapterNumber
 
     This property holds the chapter number of the media.
 
@@ -792,7 +762,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::director
+    \qmlproperty variant Video::metaData.director
 
     This property holds the director of the media.
 
@@ -800,7 +770,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::leadPerformer
+    \qmlproperty variant Video::metaData.leadPerformer
 
     This property holds the lead performer in the media.
 
@@ -808,7 +778,7 @@ QT_END_NAMESPACE
 */
 
 /*!
-    \qmlproperty variant Video::writer
+    \qmlproperty variant Video::metaData.writer
 
     This property holds the writer of the media.
 
@@ -820,157 +790,157 @@ QT_END_NAMESPACE
 #ifndef Q_QDOC
 
 /*!
-    \qmlproperty variant Video::cameraManufacturer
+    \qmlproperty variant Video::metaData.cameraManufacturer
 
     \sa {QtMultimediaKit::CameraManufacturer}
 */
 
 /*!
-    \qmlproperty variant Video::cameraModel
+    \qmlproperty variant Video::metaData.cameraModel
 
     \sa {QtMultimediaKit::CameraModel}
 */
 
 /*!
-    \qmlproperty variant Video::event
+    \qmlproperty variant Video::metaData.event
 
     \sa {QtMultimediaKit::Event}
 */
 
 /*!
-    \qmlproperty variant Video::subject
+    \qmlproperty variant Video::metaData.subject
 
     \sa {QtMultimediaKit::Subject}
 */
 
 /*!
-    \qmlproperty variant Video::orientation
+    \qmlproperty variant Video::metaData.orientation
 
     \sa {QtMultimediaKit::Orientation}
 */
 
 /*!
-    \qmlproperty variant Video::exposureTime
+    \qmlproperty variant Video::metaData.exposureTime
 
     \sa {QtMultimediaKit::ExposureTime}
 */
 
 /*!
-    \qmlproperty variant Video::fNumber
+    \qmlproperty variant Video::metaData.fNumber
 
     \sa {QtMultimediaKit::FNumber}
 */
 
 /*!
-    \qmlproperty variant Video::exposureProgram
+    \qmlproperty variant Video::metaData.exposureProgram
 
     \sa {QtMultimediaKit::ExposureProgram}
 */
 
 /*!
-    \qmlproperty variant Video::isoSpeedRatings
+    \qmlproperty variant Video::metaData.isoSpeedRatings
 
     \sa {QtMultimediaKit::ISOSpeedRatings}
 */
 
 /*!
-    \qmlproperty variant Video::exposureBiasValue
+    \qmlproperty variant Video::metaData.exposureBiasValue
 
     \sa {QtMultimediaKit::ExposureBiasValue}
 */
 
 /*!
-    \qmlproperty variant Video::dateTimeDigitized
+    \qmlproperty variant Video::metaData.dateTimeDigitized
 
     \sa {QtMultimediaKit::DateTimeDigitized}
 */
 
 /*!
-    \qmlproperty variant Video::subjectDistance
+    \qmlproperty variant Video::metaData.subjectDistance
 
     \sa {QtMultimediaKit::SubjectDistance}
 */
 
 /*!
-    \qmlproperty variant Video::meteringMode
+    \qmlproperty variant Video::metaData.meteringMode
 
     \sa {QtMultimediaKit::MeteringMode}
 */
 
 /*!
-    \qmlproperty variant Video::lightSource
+    \qmlproperty variant Video::metaData.lightSource
 
     \sa {QtMultimediaKit::LightSource}
 */
 
 /*!
-    \qmlproperty variant Video::flash
+    \qmlproperty variant Video::metaData.flash
 
     \sa {QtMultimediaKit::Flash}
 */
 
 /*!
-    \qmlproperty variant Video::focalLength
+    \qmlproperty variant Video::metaData.focalLength
 
     \sa {QtMultimediaKit::FocalLength}
 */
 
 /*!
-    \qmlproperty variant Video::exposureMode
+    \qmlproperty variant Video::metaData.exposureMode
 
     \sa {QtMultimediaKit::ExposureMode}
 */
 
 /*!
-    \qmlproperty variant Video::whiteBalance
+    \qmlproperty variant Video::metaData.whiteBalance
 
     \sa {QtMultimediaKit::WhiteBalance}
 */
 
 /*!
-    \qmlproperty variant Video::DigitalZoomRatio
+    \qmlproperty variant Video::metaData.DigitalZoomRatio
 
     \sa {QtMultimediaKit::DigitalZoomRatio}
 */
 
 /*!
-    \qmlproperty variant Video::focalLengthIn35mmFilm
+    \qmlproperty variant Video::metaData.focalLengthIn35mmFilm
 
     \sa {QtMultimediaKit::FocalLengthIn35mmFile}
 */
 
 /*!
-    \qmlproperty variant Video::sceneCaptureType
+    \qmlproperty variant Video::metaData.sceneCaptureType
 
     \sa {QtMultimediaKit::SceneCaptureType}
 */
 
 /*!
-    \qmlproperty variant Video::gainControl
+    \qmlproperty variant Video::metaData.gainControl
 
     \sa {QtMultimediaKit::GainControl}
 */
 
 /*!
-    \qmlproperty variant Video::contrast
+    \qmlproperty variant Video::metaData.contrast
 
     \sa {QtMultimediaKit::contrast}
 */
 
 /*!
-    \qmlproperty variant Video::saturation
+    \qmlproperty variant Video::metaData.saturation
 
     \sa {QtMultimediaKit::Saturation}
 */
 
 /*!
-    \qmlproperty variant Video::sharpness
+    \qmlproperty variant Video::metaData.sharpness
 
     \sa {QtMultimediaKit::Sharpness}
 */
 
 /*!
-    \qmlproperty variant Video::deviceSettingDescription
+    \qmlproperty variant Video::metaData.deviceSettingDescription
 
     \sa {QtMultimediaKit::DeviceSettingDescription}
 */
