@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -48,8 +42,10 @@
 #include "qgstreamercapturesession.h"
 #include "qgstreamerrecordercontrol.h"
 #include "qgstreamermediacontainercontrol.h"
+#include "qgstreamervideorendererinterface.h"
 #include "qgstreameraudioencode.h"
 #include "qgstreamervideoencode.h"
+#include "qgstreamerimageencode.h"
 #include "qgstreamerbushelper.h"
 #include <qmediarecorder.h>
 
@@ -78,11 +74,14 @@ QGstreamerCaptureSession::QGstreamerCaptureSession(QGstreamerCaptureSession::Cap
      m_audioInputFactory(0),
      m_audioPreviewFactory(0),
      m_videoInputFactory(0),
-     m_videoPreviewFactory(0),
+     m_viewfinder(0),
+     m_viewfinderInterface(0),
      m_audioSrc(0),
      m_audioTee(0),
      m_audioPreviewQueue(0),
      m_audioPreview(0),
+     m_audioVolume(0),
+     m_muted(false),
      m_videoSrc(0),
      m_videoTee(0),
      m_videoPreviewQueue(0),
@@ -100,7 +99,8 @@ QGstreamerCaptureSession::QGstreamerCaptureSession(QGstreamerCaptureSession::Cap
     m_busHelper->installSyncEventFilter(this);
     connect(m_busHelper, SIGNAL(message(QGstreamerMessage)), SLOT(busMessage(QGstreamerMessage)));
     m_audioEncodeControl = new QGstreamerAudioEncode(this);
-    m_videoEncodeControl = new QGstreamerVideoEncode(this);    
+    m_videoEncodeControl = new QGstreamerVideoEncode(this);
+    m_imageEncodeControl = new QGstreamerImageEncode(this);
     m_recorderControl = new QGstreamerRecorderControl(this);
     m_mediaContainerControl = new QGstreamerMediaContainerControl(this);
 
@@ -114,6 +114,11 @@ QGstreamerCaptureSession::~QGstreamerCaptureSession()
     gst_object_unref(GST_OBJECT(m_pipeline));
 }
 
+void QGstreamerCaptureSession::setCaptureMode(CaptureMode mode)
+{
+    m_captureMode = mode;
+}
+
 GstElement *QGstreamerCaptureSession::buildEncodeBin()
 {
     bool ok = true;
@@ -137,15 +142,15 @@ GstElement *QGstreamerCaptureSession::buildEncodeBin()
     if (m_captureMode & Audio) {
         GstElement *audioConvert = gst_element_factory_make("audioconvert", "audioconvert");
         GstElement *audioQueue = gst_element_factory_make("queue", "audio-encode-queue");
-        GstElement *volume = gst_element_factory_make("volume", "volume");
+        m_audioVolume = gst_element_factory_make("volume", "volume");
         GstElement *audioEncoder = m_audioEncodeControl->createEncoder();
 
         ok &= audioEncoder != 0;
 
-        gst_bin_add_many(GST_BIN(encodeBin), audioConvert, audioQueue, volume, audioEncoder,  NULL);
+        gst_bin_add_many(GST_BIN(encodeBin), audioConvert, audioQueue, m_audioVolume, audioEncoder,  NULL);
 
-        ok &= gst_element_link_many(audioConvert, audioQueue, volume, audioEncoder, muxer, NULL);
-        //g_object_set(G_OBJECT(volume), "volume", 10.0, NULL);
+        ok &= gst_element_link_many(audioConvert, audioQueue, m_audioVolume, audioEncoder, muxer, NULL);
+        g_object_set(G_OBJECT(m_audioVolume), "volume", (m_muted ? 0.0 : 1.0), NULL);
 
         // add ghostpads
         GstPad *pad = gst_element_get_static_pad(audioConvert, "sink");
@@ -270,21 +275,28 @@ GstElement *QGstreamerCaptureSession::buildVideoPreview()
 {
     GstElement *previewElement = 0;
 
-    if (m_videoPreviewFactory) {
+    if (m_viewfinderInterface) {
         GstElement *bin = gst_bin_new("video-preview-bin");
         GstElement *colorspace = gst_element_factory_make("ffmpegcolorspace", "ffmpegcolorspace-preview");
         GstElement *capsFilter = gst_element_factory_make("capsfilter", "capsfilter-video-preview");
-        GstElement *preview = m_videoPreviewFactory->buildElement();
+        GstElement *preview = m_viewfinderInterface->videoSink();
 
         gst_bin_add_many(GST_BIN(bin), colorspace, capsFilter, preview,  NULL);
         gst_element_link(colorspace,capsFilter);
         gst_element_link(capsFilter,preview);
 
-        QVideoEncoderSettings videoSettings = m_videoEncodeControl->videoSettings();
-        if (!videoSettings.resolution().isEmpty() || videoSettings.frameRate() > 0.001) {
-            QSize resolution = videoSettings.resolution();
-            qreal frameRate = videoSettings.frameRate();
+        QSize resolution;
+        qreal frameRate = 0;
 
+        if (m_captureMode & Video) {
+            QVideoEncoderSettings videoSettings = m_videoEncodeControl->videoSettings();
+            resolution = videoSettings.resolution();
+            frameRate = videoSettings.frameRate();
+        } else if (m_captureMode & Image) {
+            resolution = m_imageEncodeControl->imageSettings().resolution();
+        }
+
+        if (!resolution.isEmpty() || frameRate > 0.001) {
             GstCaps *caps = gst_caps_new_empty();
             QStringList structureTypes;
             structureTypes << "video/x-raw-yuv" << "video/x-raw-rgb";
@@ -344,6 +356,171 @@ GstElement *QGstreamerCaptureSession::buildVideoPreview()
     return previewElement;
 }
 
+
+static gboolean passImageFilter(GstElement *element,
+                                GstBuffer *buffer,
+                                void *appdata)
+{
+    Q_UNUSED(element);
+    Q_UNUSED(buffer);
+
+    QGstreamerCaptureSession *session = (QGstreamerCaptureSession *)appdata;
+    if (session->m_passImage || session->m_passPrerollImage) {
+        session->m_passImage = false;
+
+        if (session->m_passPrerollImage) {
+            session->m_passPrerollImage = false;
+            return TRUE;
+        }
+        session->m_passPrerollImage = false;
+
+        QImage img;
+
+        GstCaps *caps = gst_buffer_get_caps(buffer);
+        if (caps) {
+            GstStructure *structure = gst_caps_get_structure (caps, 0);
+            gint width = 0;
+            gint height = 0;
+
+            if (structure &&
+                gst_structure_get_int(structure, "width", &width) &&
+                gst_structure_get_int(structure, "height", &height) &&
+                width > 0 && height > 0) {
+                    if (qstrcmp(gst_structure_get_name(structure), "video/x-raw-yuv") == 0) {
+                        guint32 fourcc = 0;
+                        gst_structure_get_fourcc(structure, "format", &fourcc);
+
+                        if (fourcc == GST_MAKE_FOURCC('I','4','2','0')) {
+                            img = QImage(width/2, height/2, QImage::Format_RGB32);
+
+                            const uchar *data = (const uchar *)buffer->data;
+
+                            for (int y=0; y<height; y+=2) {
+                                const uchar *yLine = data + y*width;
+                                const uchar *uLine = data + width*height + y*width/4;
+                                const uchar *vLine = data + width*height*5/4 + y*width/4;
+
+                                for (int x=0; x<width; x+=2) {
+                                    const qreal Y = 1.164*(yLine[x]-16);
+                                    const int U = uLine[x/2]-128;
+                                    const int V = vLine[x/2]-128;
+
+                                    int b = qBound(0, int(Y + 2.018*U), 255);
+                                    int g = qBound(0, int(Y - 0.813*V - 0.391*U), 255);
+                                    int r = qBound(0, int(Y + 1.596*V), 255);
+
+                                    img.setPixel(x/2,y/2,qRgb(r,g,b));
+                                }
+                            }
+                        }
+
+                    } else if (qstrcmp(gst_structure_get_name(structure), "video/x-raw-rgb") == 0) {
+                        QImage::Format format = QImage::Format_Invalid;
+                        int bpp = 0;
+                        gst_structure_get_int(structure, "bpp", &bpp);
+
+                        if (bpp == 24)
+                            format = QImage::Format_RGB888;
+                        else if (bpp == 32)
+                            format = QImage::Format_RGB32;
+
+                        if (format != QImage::Format_Invalid) {
+                            img = QImage((const uchar *)buffer->data,
+                                         width,
+                                         height,
+                                         format);
+                            img.bits(); //detach
+                        }
+                    }
+            }
+            gst_caps_unref(caps);
+        }
+
+        static int exposedSignalIndex = session->metaObject()->indexOfSignal("imageExposed(int)");
+        session->metaObject()->method(exposedSignalIndex).invoke(session,
+                                                          Qt::QueuedConnection,
+                                                          Q_ARG(int,session->m_imageRequestId));
+
+        static int capturedSignalIndex = session->metaObject()->indexOfSignal("imageCaptured(int,QImage)");
+        session->metaObject()->method(capturedSignalIndex).invoke(session,
+                                                          Qt::QueuedConnection,
+                                                          Q_ARG(int,session->m_imageRequestId),
+                                                          Q_ARG(QImage,img));
+
+        return TRUE;
+    } else {
+        return FALSE;
+    }
+}
+
+static gboolean saveImageFilter(GstElement *element,
+                                GstBuffer *buffer,
+                                GstPad *pad,
+                                void *appdata)
+{
+    Q_UNUSED(element);
+    Q_UNUSED(pad);
+    QGstreamerCaptureSession *session = (QGstreamerCaptureSession *)appdata;
+
+    QString fileName = session->m_imageFileName;
+
+    if (!fileName.isEmpty()) {
+        QFile f(fileName);
+        if (f.open(QFile::WriteOnly)) {
+            f.write((const char *)buffer->data, buffer->size);
+            f.close();
+
+            static int signalIndex = session->metaObject()->indexOfSignal("imageSaved(int,QString)");
+            session->metaObject()->method(signalIndex).invoke(session,
+                                                              Qt::QueuedConnection,
+                                                              Q_ARG(int,session->m_imageRequestId),
+                                                              Q_ARG(QString,fileName));
+        }
+    }
+
+    return TRUE;
+}
+
+GstElement *QGstreamerCaptureSession::buildImageCapture()
+{
+    GstElement *bin = gst_bin_new("image-capture-bin");
+    GstElement *queue = gst_element_factory_make("queue", "queue-image-capture");
+    GstElement *colorspace = gst_element_factory_make("ffmpegcolorspace", "ffmpegcolorspace-image-capture");
+    GstElement *encoder = gst_element_factory_make("jpegenc", "image-encoder");
+    GstElement *sink = gst_element_factory_make("fakesink","sink-image-capture");
+
+    GstPad *pad = gst_element_get_static_pad(queue, "src");
+    Q_ASSERT(pad);
+    gst_pad_add_buffer_probe(pad, G_CALLBACK(passImageFilter), this);
+
+    g_object_set(G_OBJECT(sink), "signal-handoffs", TRUE, NULL);
+    g_signal_connect(G_OBJECT(sink), "handoff",
+                     G_CALLBACK(saveImageFilter), this);
+
+    gst_bin_add_many(GST_BIN(bin), queue, colorspace, encoder, sink,  NULL);
+    gst_element_link_many(queue, colorspace, encoder, sink, NULL);
+
+    // add ghostpads
+    pad = gst_element_get_static_pad(queue, "sink");
+    Q_ASSERT(pad);
+    gst_element_add_pad(GST_ELEMENT(bin), gst_ghost_pad_new("imagesink", pad));
+    gst_object_unref(GST_OBJECT(pad));
+
+    m_passImage = false;
+    m_passPrerollImage = true;
+    m_imageFileName = QString();
+
+    return bin;
+}
+
+void QGstreamerCaptureSession::captureImage(int requestId, const QString &fileName)
+{
+    m_imageRequestId = requestId;
+    m_imageFileName = fileName;
+    m_passImage = true;
+}
+
+
 #define REMOVE_ELEMENT(element) { if (element) {gst_bin_remove(GST_BIN(m_pipeline), element); element = 0;} }
 
 bool QGstreamerCaptureSession::rebuildGraph(QGstreamerCaptureSession::PipelineMode newMode)
@@ -357,6 +534,8 @@ bool QGstreamerCaptureSession::rebuildGraph(QGstreamerCaptureSession::PipelineMo
     REMOVE_ELEMENT(m_videoPreviewQueue);
     REMOVE_ELEMENT(m_videoTee);
     REMOVE_ELEMENT(m_encodeBin);
+    REMOVE_ELEMENT(m_imageCaptureBin);
+    m_audioVolume = 0;
 
     bool ok = true;
 
@@ -375,21 +554,24 @@ bool QGstreamerCaptureSession::rebuildGraph(QGstreamerCaptureSession::PipelineMo
                     ok &= gst_element_link(m_audioSrc, m_audioPreview);
                 }
             }
-            if (m_captureMode & Video) {
+            if (m_captureMode & Video || m_captureMode & Image) {
                 m_videoSrc = buildVideoSrc();
                 m_videoTee = gst_element_factory_make("tee", "video-preview-tee");
                 m_videoPreviewQueue = gst_element_factory_make("queue", "video-preview-queue");
-                m_videoPreview = buildVideoPreview();                
+                m_videoPreview = buildVideoPreview();
+                m_imageCaptureBin = buildImageCapture();
 
-                ok &= m_videoSrc && m_videoTee && m_videoPreviewQueue && m_videoPreview;
+                ok &= m_videoSrc && m_videoTee && m_videoPreviewQueue && m_videoPreview && m_imageCaptureBin;
 
                 if (ok) {
                     gst_bin_add_many(GST_BIN(m_pipeline), m_videoSrc, m_videoTee,
-                                     m_videoPreviewQueue, m_videoPreview, NULL);
+                                     m_videoPreviewQueue, m_videoPreview,
+                                     m_imageCaptureBin, NULL);
 
                     ok &= gst_element_link(m_videoSrc, m_videoTee);
                     ok &= gst_element_link(m_videoTee, m_videoPreviewQueue);
                     ok &= gst_element_link(m_videoPreviewQueue, m_videoPreview);
+                    ok &= gst_element_link(m_videoTee, m_imageCaptureBin);
                 }
             }
             break;
@@ -442,7 +624,7 @@ bool QGstreamerCaptureSession::rebuildGraph(QGstreamerCaptureSession::PipelineMo
                 }
             }
 
-            if (ok && (m_captureMode & Video)) {
+            if (ok && (m_captureMode & Video || m_captureMode & Image)) {
                 m_videoSrc = buildVideoSrc();
                 m_videoPreview = buildVideoPreview();
                 m_videoTee = gst_element_factory_make("tee", NULL);
@@ -456,13 +638,16 @@ bool QGstreamerCaptureSession::rebuildGraph(QGstreamerCaptureSession::PipelineMo
                     ok &= gst_element_link(m_videoSrc, m_videoTee);
                     ok &= gst_element_link(m_videoTee, m_videoPreviewQueue);
                     ok &= gst_element_link(m_videoPreviewQueue, m_videoPreview);
-                    ok &= gst_element_link(m_videoTee, m_encodeBin);
                 }
+
+                if (ok && (m_captureMode & Video))
+                    ok &= gst_element_link(m_videoTee, m_encodeBin);
             }
 
             if (!m_metaData.isEmpty())
                 setMetaData(m_metaData);
 
+
             break;
     }
 
@@ -532,9 +717,41 @@ void QGstreamerCaptureSession::setVideoInput(QGstreamerVideoInput *videoInput)
     m_videoInputFactory = videoInput;
 }
 
-void QGstreamerCaptureSession::setVideoPreview(QGstreamerElementFactory *videoPreview)
+void QGstreamerCaptureSession::setVideoPreview(QObject *viewfinder)
 {
-    m_videoPreviewFactory = videoPreview;
+    m_viewfinderInterface = qobject_cast<QGstreamerVideoRendererInterface*>(viewfinder);
+    if (!m_viewfinderInterface)
+        viewfinder = 0;
+
+    if (m_viewfinder != viewfinder) {
+        bool oldReady = isReady();
+
+        if (m_viewfinder) {
+            disconnect(m_viewfinder, SIGNAL(sinkChanged()),
+                       this, SIGNAL(viewfinderChanged()));
+            disconnect(m_viewfinder, SIGNAL(readyChanged(bool)),
+                       this, SIGNAL(readyChanged(bool)));
+        }
+
+        m_viewfinder = viewfinder;
+        //m_viewfinderHasChanged = true;
+
+        if (m_viewfinder) {
+            connect(m_viewfinder, SIGNAL(sinkChanged()),
+                       this, SIGNAL(viewfinderChanged()));
+            connect(m_viewfinder, SIGNAL(readyChanged(bool)),
+                    this, SIGNAL(readyChanged(bool)));
+        }
+
+        emit viewfinderChanged();
+        if (oldReady != isReady())
+            emit readyChanged(isReady());
+    }
+}
+
+bool QGstreamerCaptureSession::isReady() const
+{
+    return m_viewfinderInterface != 0 && m_viewfinderInterface->isReady();
 }
 
 QGstreamerCaptureSession::State QGstreamerCaptureSession::state() const
@@ -703,8 +920,8 @@ bool QGstreamerCaptureSession::processSyncMessage(const QGstreamerMessage &messa
             if (m_audioPreviewFactory)
                 m_audioPreviewFactory->prepareWinId();
 
-            if (m_videoPreviewFactory)
-                m_videoPreviewFactory->prepareWinId();
+            if (m_viewfinderInterface)
+                m_viewfinderInterface->precessNewStream();
 
             return true;
         }
@@ -803,3 +1020,12 @@ void QGstreamerCaptureSession::busMessage(const QGstreamerMessage &message)
     }
 }
 
+void QGstreamerCaptureSession::setMuted(bool muted)
+{
+    if (m_muted != muted) {
+        m_muted = muted;
+        if (m_audioVolume)
+            g_object_set(G_OBJECT(m_audioVolume), "volume", (m_muted ? 0.0 : 1.0), NULL);
+        emit mutedChanged(muted);
+    }
+}
