@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -61,6 +55,7 @@
 
 #include "qvideosurfacegstsink.h"
 
+//#define DEBUG_VIDEO_SURFACE_SINK
 
 
 Q_DECLARE_METATYPE(QVideoSurfaceFormat)
@@ -72,6 +67,7 @@ QVideoSurfaceGstDelegate::QVideoSurfaceGstDelegate(QAbstractVideoSurface *surfac
 {
     if (m_surface) {
         m_supportedPixelFormats = m_surface->supportedPixelFormats();
+        m_supportedXVideoPixelFormats = m_surface->supportedPixelFormats(QAbstractVideoBuffer::XvShmImageHandle);
         connect(m_surface, SIGNAL(supportedFormatsChanged()), this, SLOT(supportedFormatsChanged()));
     }
 }
@@ -80,8 +76,12 @@ QList<QVideoFrame::PixelFormat> QVideoSurfaceGstDelegate::supportedPixelFormats(
 {
     QMutexLocker locker(const_cast<QMutex *>(&m_mutex));
 
-    if (handleType == QAbstractVideoBuffer::NoHandle || !m_surface)
+    if (!m_surface)
+        return QList<QVideoFrame::PixelFormat>();
+    else if (handleType == QAbstractVideoBuffer::NoHandle)
         return m_supportedPixelFormats;
+    else if (handleType == QAbstractVideoBuffer::XvShmImageHandle)
+        return m_supportedXVideoPixelFormats;
     else
         return m_surface->supportedPixelFormats(handleType);
 }
@@ -137,13 +137,16 @@ void QVideoSurfaceGstDelegate::stop()
 bool QVideoSurfaceGstDelegate::isActive()
 {
     QMutexLocker locker(&m_mutex);
-    return m_surface->isActive();
+    return !m_surface.isNull() && m_surface->isActive();
 }
 
 GstFlowReturn QVideoSurfaceGstDelegate::render(GstBuffer *buffer)
 {
-    if (!m_surface)
-        return GST_FLOW_NOT_NEGOTIATED;
+    if (!m_surface) {
+        qWarning() << "Rendering video frame to deleted surface, skip.";
+        //return GST_FLOW_NOT_NEGOTIATED;
+        return GST_FLOW_OK;
+    }
 
     QMutexLocker locker(&m_mutex);
 
@@ -153,7 +156,7 @@ GstFlowReturn QVideoSurfaceGstDelegate::render(GstBuffer *buffer)
     if (G_TYPE_CHECK_INSTANCE_TYPE(buffer, QGstXvImageBuffer::get_type())) {
         QGstXvImageBuffer *xvBuffer = reinterpret_cast<QGstXvImageBuffer *>(buffer);
         QVariant handle = QVariant::fromValue(xvBuffer->xvImage);
-        videoBuffer = new QGstVideoBuffer(buffer, m_bytesPerLine, XvHandleType, handle);
+        videoBuffer = new QGstVideoBuffer(buffer, m_bytesPerLine, QAbstractVideoBuffer::XvShmImageHandle, handle);
     } else
 #endif
         videoBuffer = new QGstVideoBuffer(buffer, m_bytesPerLine);
@@ -208,7 +211,8 @@ void QVideoSurfaceGstDelegate::queuedRender()
     QMutexLocker locker(&m_mutex);
 
     if (m_surface.isNull()) {
-        m_renderReturn = GST_FLOW_ERROR;
+        qWarning() << "Rendering video frame to deleted surface, skip the frame";
+        m_renderReturn = GST_FLOW_OK;
     } else if (m_surface->present(m_frame)) {
         m_renderReturn = GST_FLOW_OK;
     } else {
@@ -217,10 +221,13 @@ void QVideoSurfaceGstDelegate::queuedRender()
             m_renderReturn = GST_FLOW_OK;
             break;
         case QAbstractVideoSurface::StoppedError:
-            m_renderReturn = GST_FLOW_NOT_NEGOTIATED;
+            //It's likely we are in process of changing video output
+            //and the surface is already stopped, ignore the frame
+            m_renderReturn = GST_FLOW_OK;
             break;
         default:
-            m_renderReturn = GST_FLOW_ERROR;
+            qWarning() << "Failed to render video frame:" << m_surface->error();
+            m_renderReturn = GST_FLOW_OK;
             break;
         }
     }
@@ -233,8 +240,11 @@ void QVideoSurfaceGstDelegate::supportedFormatsChanged()
     QMutexLocker locker(&m_mutex);
 
     m_supportedPixelFormats.clear();
-    if (m_surface)
+    m_supportedXVideoPixelFormats.clear();
+    if (m_surface) {
         m_supportedPixelFormats = m_surface->supportedPixelFormats();
+        m_supportedXVideoPixelFormats = m_surface->supportedPixelFormats(QAbstractVideoBuffer::XvShmImageHandle);
+    }
 }
 
 struct YuvFormat
@@ -499,8 +509,10 @@ gboolean QVideoSurfaceGstSink::set_caps(GstBaseSink *base, GstCaps *caps)
 {
     VO_SINK(base);
 
-    //qDebug() << "set_caps";
-    //qDebug() << gst_caps_to_string(caps);
+#ifdef DEBUG_VIDEO_SURFACE_SINK
+    qDebug() << "set_caps:";
+    qDebug() << gst_caps_to_string(caps);
+#endif
 
     if (!caps) {
         sink->delegate->stop();
@@ -524,13 +536,16 @@ gboolean QVideoSurfaceGstSink::set_caps(GstBaseSink *base, GstCaps *caps)
             gst_caps_unref(sink->lastRequestedCaps);
         sink->lastRequestedCaps = 0;
 
-        //qDebug() << "Staring video surface:";
-        //qDebug() << format;
-        //qDebug() << bytesPerLine;
+#ifdef DEBUG_VIDEO_SURFACE_SINK
+        qDebug() << "Staring video surface, format:";
+        qDebug() << format;
+        qDebug() << "bytesPerLine:" << bytesPerLine;
+#endif
 
         if (sink->delegate->start(format, bytesPerLine))
             return TRUE;
-
+        else
+            qWarning() << "Failed to start video surface";
     }
 
     return FALSE;
@@ -623,7 +638,7 @@ GstFlowReturn QVideoSurfaceGstSink::buffer_alloc(
         return GST_FLOW_OK;
     }
 
-    if (sink->delegate->supportedPixelFormats(XvHandleType).isEmpty()) {
+    if (sink->delegate->supportedPixelFormats(QAbstractVideoBuffer::XvShmImageHandle).isEmpty()) {
         //qDebug() << "sink doesn't support Xv buffers, skip buffers allocation";
         return GST_FLOW_OK;
     }
@@ -642,7 +657,9 @@ GstFlowReturn QVideoSurfaceGstSink::buffer_alloc(
 
         if (format.pixelFormat() != surfaceFormat.pixelFormat() ||
             format.frameSize() != surfaceFormat.frameSize()) {
-            //qDebug() << "new format requested, restart video surface";
+#ifdef DEBUG_VIDEO_SURFACE_SINK
+            qDebug() << "new format requested, restart video surface";
+#endif
             sink->delegate->stop();
         }
     }
@@ -652,7 +669,7 @@ GstFlowReturn QVideoSurfaceGstSink::buffer_alloc(
         QVideoSurfaceFormat format = formatForCaps(intersection, &bytesPerLine);
 
         if (!sink->delegate->start(format, bytesPerLine)) {
-            //qDebug() << "failed to start video surface";
+            qWarning() << "failed to start video surface";
             return GST_FLOW_NOT_NEGOTIATED;
         }
     }
@@ -714,7 +731,6 @@ gboolean QVideoSurfaceGstSink::event(GstBaseSink *base, GstEvent *event)
 GstFlowReturn QVideoSurfaceGstSink::preroll(GstBaseSink *base, GstBuffer *buffer)
 {
     VO_SINK(base);
-
     return sink->delegate->render(buffer);
 }
 
