@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -50,7 +44,6 @@
 #include "s60mediaplayerservice.h"
 #include "s60videooverlay.h"
 
-#include <QtCore/qdebug.h>
 #include <QtGui/qwidget.h>
 #include <QtCore/qtimer.h>
 #include <QApplication>
@@ -81,7 +74,6 @@ S60VideoPlayerSession::S60VideoPlayerSession(QMediaService *service)
     m_audioOutput = 0;
 #endif
 
-    resetNativeHandles();
 #ifdef MMF_VIDEO_SURFACES_SUPPORTED
     QT_TRAP_THROWING(m_player = CVideoPlayerUtility2::NewL(
         *this,
@@ -89,6 +81,7 @@ S60VideoPlayerSession::S60VideoPlayerSession(QMediaService *service)
         EMdaPriorityPreferenceNone
         ));
 #else
+    resetNativeHandles();
     QT_TRAP_THROWING(m_player = CVideoPlayerUtility::NewL(
         *this,
         0,
@@ -161,10 +154,12 @@ void S60VideoPlayerSession::setVideoRenderer(QObject *videoOutput)
 
     if (oldWidgetControl) {
         disconnect(oldWidgetControl, SIGNAL(widgetUpdated()), this, SLOT(resetVideoDisplay()));
+        disconnect(oldWidgetControl, SIGNAL(widgetResized()), this, SLOT(resizeVideoWindow()));
         disconnect(this, SIGNAL(stateChanged(QMediaPlayer::State)), oldWidgetControl, SLOT(videoStateChanged(QMediaPlayer::State)));
     }
     if (newWidgetControl) {
         connect(newWidgetControl, SIGNAL(widgetUpdated()), this, SLOT(resetVideoDisplay()));
+        connect(newWidgetControl, SIGNAL(widgetResized()), this, SLOT(resizeVideoWindow()));
         connect(this, SIGNAL(stateChanged(QMediaPlayer::State)), newWidgetControl, SLOT(videoStateChanged(QMediaPlayer::State)));
     }
 
@@ -181,7 +176,6 @@ void S60VideoPlayerSession::setVideoRenderer(QObject *videoOutput)
     S60VideoWidgetControl *oldWidgetControl = qobject_cast<S60VideoWidgetControl *>(m_videoOutput);
 
     if (oldWidgetControl) {
-        S60VideoWidgetControl *oldWidgetControl = qobject_cast<S60VideoWidgetControl *>(m_videoOutput);
         disconnect(oldWidgetControl, SIGNAL(widgetUpdated()), this, SLOT(resetVideoDisplay()));
         disconnect(oldWidgetControl, SIGNAL(beginVideoWindowNativePaint()), this, SLOT(suspendDirectScreenAccess()));
         disconnect(oldWidgetControl, SIGNAL(endVideoWindowNativePaint()), this, SLOT(resumeDirectScreenAccess()));
@@ -204,25 +198,23 @@ bool S60VideoPlayerSession::resetNativeHandles()
     TRect newRect = TRect(0,0,0,0);
     Qt::AspectRatioMode aspectRatioMode = Qt::KeepAspectRatio;
 
-    S60VideoWidgetControl* widgetControl = qobject_cast<S60VideoWidgetControl *>(m_service.requestControl(QVideoWidgetControl_iid));
-    S60VideoOverlay* windowControl;
-    if (!widgetControl)
-        windowControl = qobject_cast<S60VideoOverlay *>(m_service.requestControl(QVideoWindowControl_iid));
+    S60VideoWidgetControl *widgetControl;
+
+    if (!m_videoOutput) // this is for pre Symbian^3 devices as we need output for CVideoPlayerUtility
+        widgetControl = qobject_cast<S60VideoWidgetControl *>(m_service.requestControl(QVideoWidgetControl_iid));
+    else
+        widgetControl = qobject_cast<S60VideoWidgetControl *>(m_videoOutput);
+
     if (widgetControl) {
         QWidget *videoWidget = widgetControl->videoWidget();
         newId = widgetControl->videoWidgetWId();
         newRect = QRect2TRect(QRect(videoWidget->mapToGlobal(videoWidget->pos()), videoWidget->size()));
         aspectRatioMode = widgetControl->aspectRatioMode();
-    } else if (windowControl) {
-        newId = windowControl->winId();
-        newRect = TRect( newId->DrawableWindow()->AbsPosition(), newId->DrawableWindow()->Size());
     } else {
         if (QApplication::activeWindow())
             newId = QApplication::activeWindow()->effectiveWinId();
-
         if (!newId && QApplication::allWidgets().count())
             newId = QApplication::allWidgets().at(0)->effectiveWinId();
-
         Q_ASSERT(newId != 0);
     }
     if (newRect == m_rect &&  newId == m_windowId && aspectRatioMode == m_aspectRatioMode)
@@ -238,24 +230,32 @@ bool S60VideoPlayerSession::resetNativeHandles()
     return false;
 }
 
-bool S60VideoPlayerSession::isVideoAvailable() const
+bool S60VideoPlayerSession::isVideoAvailable()
 {
 #ifdef PRE_S60_50_PLATFORM
-    return true; // this is not support in pre 5th platforms
+    return true; // this is not supported in pre 5th platforms
 #else
-    if (m_player)
-        return m_player->VideoEnabledL();
-    else
+    if (m_player) {
+        bool videoAvailable = true;
+        TRAPD(err, videoAvailable = m_player->VideoEnabledL());
+        setError(err);
+        return videoAvailable;
+    }else {
         return false;
+    }
 #endif
 }
 
-bool S60VideoPlayerSession::isAudioAvailable() const
+bool S60VideoPlayerSession::isAudioAvailable()
 {
-    if (m_player)
-        return m_player->AudioEnabledL();
-    else
+    if (m_player) {
+        bool audioAvailable = true;
+        TRAPD(err, audioAvailable = m_player->AudioEnabledL());
+        setError(err);
+        return audioAvailable;
+    }else {
         return false;
+    }
 }
 
 void S60VideoPlayerSession::doPlay()
@@ -285,7 +285,7 @@ void S60VideoPlayerSession::doSetPositionL(qint64 microSeconds)
 
 void S60VideoPlayerSession::doSetVolumeL(int volume)
 {
-    m_player->SetVolumeL((volume / 100.0)* m_player->MaxVolume());
+    m_player->SetVolumeL(volume * m_player->MaxVolume() / 100);
 }
 
 QPair<qreal, qreal> S60VideoPlayerSession::scaleFactor()
@@ -335,27 +335,42 @@ void S60VideoPlayerSession::MvpuoOpenComplete(TInt aError)
 #ifdef MMF_VIDEO_SURFACES_SUPPORTED
 void S60VideoPlayerSession::MvpuoPrepareComplete(TInt aError)
 {
-    setError(aError);
-    TRect rect;
-    S60VideoWidgetControl* widgetControl = qobject_cast<S60VideoWidgetControl *>(m_service.requestControl(QVideoWidgetControl_iid));
-    const QSize size = widgetControl->videoWidgetSize();
-    rect.SetSize(TSize(size.width(), size.height()));
+    setError(aError); // if we have some playback errors, handle them
 
-    if (m_displayWindow)
+    if (m_displayWindow) {
         m_player->RemoveDisplayWindow(*m_displayWindow);
+        m_displayWindow = NULL;
+    }
 
     RWindow *window = static_cast<RWindow *>(m_window);
     if (window) {
+        /* Get the window size */
+        TRect rect;
+        if (m_videoOutput && m_videoOutput->inherits("S60VideoWidgetControl")) {
+            S60VideoWidgetControl* widgetControl = qobject_cast<S60VideoWidgetControl *>(m_videoOutput);
+            const QSize size = widgetControl->videoWidgetSize();
+            rect.SetSize(TSize(size.width(), size.height()));
+        }
+        else if (m_videoOutput && m_videoOutput->inherits("S60VideoOverlay")) {
+            S60VideoOverlay* videoOverlay = qobject_cast<S60VideoOverlay *>(m_videoOutput);
+            const QSize size = videoOverlay->displayRect().size();
+            rect.SetSize(TSize(size.width(), size.height()));
+        }
+        m_rect = rect;
+
         window->SetBackgroundColor(TRgb(0, 0, 0, 255));
         TRAPD(error,
-            m_player->AddDisplayWindowL(m_wsSession, m_screenDevice, *window, rect, rect);
-            TSize originalSize;
+            m_player->AddDisplayWindowL(m_wsSession, m_screenDevice, *window, m_rect, m_rect);)
+        setError(error); // if we can't add window it an error at this point
+        TSize originalSize;
+        TRAP_IGNORE(
             m_player->VideoFrameSizeL(originalSize);
             m_originalSize = QSize(originalSize.iWidth, originalSize.iHeight);
-            m_player->SetScaleFactorL(*window, scaleFactor().first, scaleFactor().second));
-        setError(error);
+            m_player->SetScaleFactorL(*window, scaleFactor().first, scaleFactor().second);)
+
+        m_displayWindow = window;
     }
-    m_displayWindow = window;
+
 #ifdef HAS_AUDIOROUTING_IN_VIDEOPLAYER
     TRAPD(err,
         m_audioOutput = CAudioOutput::NewL(*m_player);
@@ -370,6 +385,7 @@ void S60VideoPlayerSession::MvpuoPrepareComplete(TInt aError)
 void S60VideoPlayerSession::MvpuoPrepareComplete(TInt aError)
 {
     setError(aError);
+
     TRAPD(err,
         m_player->SetDisplayWindowL(m_wsSession,
                                     m_screenDevice,
@@ -432,24 +448,33 @@ void S60VideoPlayerSession::updateMetaDataEntriesL()
 void S60VideoPlayerSession::resetVideoDisplay()
 {
     if (resetNativeHandles()) {
+
+        S60VideoWidgetControl *widgetControl = qobject_cast<S60VideoWidgetControl *>(m_videoOutput);
+        if (!widgetControl)
+            return;
+
         TRect rect;
-        S60VideoWidgetControl* widgetControl = qobject_cast<S60VideoWidgetControl *>(m_service.requestControl(QVideoWidgetControl_iid));
         const QSize size = widgetControl->videoWidgetSize();
         rect.SetSize(TSize(size.width(), size.height()));
-        if (m_displayWindow)
+        m_rect = rect;
+
+        if (m_displayWindow) {
             m_player->RemoveDisplayWindow(*m_displayWindow);
+            m_displayWindow = NULL;
+        }
+
         RWindow *window = static_cast<RWindow *>(m_window);
         if (window) {
             window->SetBackgroundColor(TRgb(0, 0, 0, 255));
-            TRAPD(err,
+            TRAP_IGNORE(
                m_player->AddDisplayWindowL(m_wsSession,
                                            m_screenDevice,
                                            *window,
-                                           rect,
-                                           rect));
-            setError(err);
+                                           m_rect,
+                                           m_rect));
+            m_displayWindow = window;
         }
-        m_displayWindow = window;
+
         if(    mediaStatus() == QMediaPlayer::LoadedMedia
             || mediaStatus() == QMediaPlayer::StalledMedia
             || mediaStatus() == QMediaPlayer::BufferingMedia
@@ -486,6 +511,33 @@ void S60VideoPlayerSession::resetVideoDisplay()
 }
 #endif //MMF_VIDEO_SURFACES_SUPPORTED
 
+void S60VideoPlayerSession::resizeVideoWindow()
+{
+#ifdef MMF_VIDEO_SURFACES_SUPPORTED
+    S60VideoWidgetControl *widgetControl = qobject_cast<S60VideoWidgetControl *>(m_videoOutput);
+    if (!widgetControl)
+        return;
+
+    m_aspectRatioMode = widgetControl->aspectRatioMode();
+
+    TRect rect;
+    const QSize size = widgetControl->videoWidgetSize();
+    rect.SetSize(TSize(size.width(), size.height()));
+    m_rect = rect;
+
+    TRAPD( err, m_player->SetVideoExtentL(*m_displayWindow, m_rect);
+        m_player->SetWindowClipRectL(*m_displayWindow, m_rect);)
+
+    // don't waste time on calling this when we have error
+    if (KErrNone != err) {
+        TSize originalSize;
+        TRAP_IGNORE(
+            m_player->VideoFrameSizeL(originalSize);
+            m_originalSize = QSize(originalSize.iWidth, originalSize.iHeight);
+            m_player->SetScaleFactorL(*m_displayWindow, scaleFactor().first, scaleFactor().second); )
+    }
+#endif //MMF_VIDEO_SURFACES_SUPPORTED
+}
 void S60VideoPlayerSession::suspendDirectScreenAccess()
 {
     m_dsaStopped = stopDirectScreenAccess();
@@ -558,11 +610,6 @@ void S60VideoPlayerSession::setActiveEndpoint(const QString& name)
     if (m_audioOutput) {
         TRAPD(err, m_audioOutput->SetAudioOutputL(output));
         setError(err);
-
-        if (m_audioEndpoint != name) {
-            m_audioEndpoint = name;
-            emit activeEndpointChanged(name);
-        }
     }
 #endif
 }
