@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -49,13 +43,16 @@
 
 char const * const maemo6magnetometer::id("maemo6.magnetometer");
 bool maemo6magnetometer::m_initDone = false;
+const float maemo6magnetometer::NANO = 0.000000001;
+
 
 maemo6magnetometer::maemo6magnetometer(QSensor *sensor)
     : maemo6sensorbase(sensor)
 {
     const QString sensorName = "magnetometersensor";
     initSensor<MagnetometerSensorChannelInterface>(sensorName, m_initDone);
-
+    setDescription(QLatin1String("magnetic flux density in teslas (T)"));
+    setRanges(NANO);
 
     if (m_sensorInterface){
         if (!(QObject::connect(m_sensorInterface, SIGNAL(dataAvailable(const MagneticField&)),
@@ -65,6 +62,7 @@ maemo6magnetometer::maemo6magnetometer(QSensor *sensor)
     else
         qWarning() << "Unable to initialize "<< sensorName;
 
+
     setReading<QMagnetometerReading>(&m_reading);
 
 }
@@ -78,17 +76,13 @@ void maemo6magnetometer::start(){
 void maemo6magnetometer::slotDataAvailable(const MagneticField& data)
 {
 
-    if (m_isGeoMagnetometer){
-        m_reading.setX( 0.0000003 * data.x() );
-        m_reading.setY( 0.0000003 * data.y() );
-        m_reading.setZ( 0.0000003 * data.z() );
-        m_reading.setCalibrationLevel( ((float) data.level()) / 3.0 );
-    } else {
-        m_reading.setX( 0.0000003 * data.rx() );
-        m_reading.setY( 0.0000003 * data.ry() );
-        m_reading.setZ( 0.0000003 * data.rz() );
-        m_reading.setCalibrationLevel(1);
-    }
+    //nanoTeslas given, divide with 10^9 to get Teslas
+    m_reading.setX( NANO * m_isGeoMagnetometer?data.x():data.rx());
+    m_reading.setY( NANO * m_isGeoMagnetometer?data.y():data.ry());
+    m_reading.setZ( NANO * m_isGeoMagnetometer?data.z():data.rz());
+    m_reading.setCalibrationLevel( m_isGeoMagnetometer?((float) data.level()) / 3.0 :1);
+
+
     m_reading.setTimestamp(data.timestamp());
     newReadingAvailable();
 }
