@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -214,7 +208,7 @@ void QNetworkSessionPrivate::updateIdentifier(const QString &newId)
 }
 
 
-quint64 QNetworkSessionPrivate::getStatistics(bool sent) const
+QNetworkSessionPrivate::Statistics QNetworkSessionPrivate::getStatistics() const
 {
     /* This could be also implemented by using the Maemo::Icd::statistics()
      * that gets the statistics data for a specific IAP. Change if
@@ -222,52 +216,47 @@ quint64 QNetworkSessionPrivate::getStatistics(bool sent) const
      */
     Maemo::Icd icd;
     QList<Maemo::IcdStatisticsResult> stats_results;
-    quint64 counter_rx = 0, counter_tx = 0;
+    Statistics stats = { 0, 0, 0};
 
-    if (!icd.statistics(stats_results)) {
-	return 0;
-    }
+    if (!icd.statistics(stats_results))
+        return stats;
 
     foreach (const Maemo::IcdStatisticsResult &res, stats_results) {
-	if (res.params.network_attrs & ICD_NW_ATTR_IAPNAME) {
-	    /* network_id is the IAP UUID */
-	    if (QString(res.params.network_id.data()) == activeConfig.identifier()) {
-		counter_tx = res.bytes_sent;
-		counter_rx = res.bytes_received;
-	    }
-	} else {
-	    /* We probably will never get to this branch */
-	    QNetworkConfigurationPrivate *d = activeConfig.d.data();
-	    if (res.params.network_id == d->network_id) {
-		counter_tx = res.bytes_sent;
-		counter_rx = res.bytes_received;
-	    }
-	}
+        if (res.params.network_attrs & ICD_NW_ATTR_IAPNAME) {
+            /* network_id is the IAP UUID */
+            if (QString(res.params.network_id.data()) == activeConfig.identifier()) {
+                stats.txData = res.bytes_sent;
+                stats.rxData = res.bytes_received;
+                stats.activeTime = res.time_active;
+            }
+        } else {
+            /* We probably will never get to this branch */
+            QNetworkConfigurationPrivate *d = activeConfig.d.data();
+            if (res.params.network_id == d->network_id) {
+                stats.txData = res.bytes_sent;
+                stats.rxData = res.bytes_received;
+                stats.activeTime = res.time_active;
+            }
+        }
     }
 
-    if (sent)
-	return counter_tx;
-    else
-	return counter_rx;
+    return stats;
 }
 
 
 quint64 QNetworkSessionPrivate::bytesWritten() const
 {
-    return getStatistics(true);
+    return getStatistics().txData;
 }
 
 quint64 QNetworkSessionPrivate::bytesReceived() const
 {
-    return getStatistics(false);
+    return getStatistics().rxData;
 }
 
 quint64 QNetworkSessionPrivate::activeTime() const
 {
-    if (startTime.isNull()) {
-        return 0;
-    }
-    return startTime.secsTo(QDateTime::currentDateTime());
+    return getStatistics().activeTime;
 }
 
 
@@ -749,7 +738,7 @@ void QNetworkSessionPrivate::do_open()
             qDebug() << "connect to"<< publicConfig.identifier() << "failed, result is empty";
 #endif
             updateState(QNetworkSession::Disconnected);
-            emit q->error(QNetworkSession::InvalidConfigurationError);
+            emit q->error(QNetworkSession::SessionAbortedError);
             if (publicConfig.type() == QNetworkConfiguration::UserChoice)
                     copyConfig(publicConfig, activeConfig);
             return;
@@ -763,7 +752,7 @@ void QNetworkSessionPrivate::do_open()
         if ((publicConfig.type() != QNetworkConfiguration::UserChoice) &&
             (connected_iap != config.identifier())) {
             updateState(QNetworkSession::Disconnected);
-            emit q->error(QNetworkSession::InvalidConfigurationError);
+            emit q->error(QNetworkSession::UnknownSessionError);
             return;
         }
 
@@ -1001,6 +990,9 @@ QString QNetworkSessionPrivate::errorString() const
     case QNetworkSession::SessionAbortedError:
         errorStr = QObject::tr("Session aborted by user or system");
         break;
+    case QNetworkSession::InvalidConfigurationError:
+        errorStr = QObject::tr("The specified configuration cannot be used.");
+        break;
     default:
     case QNetworkSession::UnknownSessionError:
         errorStr = QObject::tr("Unidentified Error");
