@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -49,6 +43,7 @@
 #include "qcontactdetail_p.h"
 #include "qcontactmanager.h"
 #include <QDebug>
+#include <QDataStream>
 
 QTM_BEGIN_NAMESPACE
 
@@ -79,12 +74,13 @@ static int qClearAllocatedStringHash()
     QContactStringHolder::s_qstrings.clear();
     return 1;
 }
-Q_DESTRUCTOR_FUNCTION(qClearAllocatedStringHash);
+Q_DESTRUCTOR_FUNCTION(qClearAllocatedStringHash)
 
 /*!
   \class QContactDetail
  
   \brief The QContactDetail class represents a single, complete detail about a contact.
+  \inmodule QtContacts
   \ingroup contacts-main
  
   All of the information for a contact is stored in one or more QContactDetail objects.
@@ -190,97 +186,6 @@ Q_DESTRUCTOR_FUNCTION(qClearAllocatedStringHash);
   \snippet ../../src/contacts/details/qcontactphonenumber.h 0
  */
 
-/*!
-    \class QLatin1Constant
-    \headerfile
-    \brief The QLatin1Constant class holds a Latin 1 string constant
-
-*/
-
-/*!
-    \fn QLatin1Constant::operator QString() const
-    \internal
- */
-/*!
-    \fn QLatin1Constant::operator QLatin1String() const
-    \internal
- */
-/*!
-    \fn QLatin1Constant::operator QVariant() const
-    \internal
- */
-/*!
-    \fn bool QLatin1Constant::operator ==(const QLatin1Constant& other) const
-
-    Returns true if this QLatin1Constant is the same as \a other (either same object or
-    same string literal), and false otherwise.
- */
-/*!
-    \fn bool QLatin1Constant::operator !=(const QLatin1Constant& other) const
-
-    Returns false if this QLatin1Constant is the same as \a other (either same object or
-    same string literal), and true otherwise.
-*/
-/*!
-    \fn inline const char * QLatin1Constant::latin1() const
-
-    Returns the value of this literal as a C style string (null terminated).
-*/
-
-
-/*!
-  \macro Q_DECLARE_LATIN1_CONSTANT
-  \relates QLatin1Constant
-
-  This macro, along with the related Q_DEFINE_LATIN1_CONSTANT macro,
-  allows you to describe a "Latin 1 string constant".
-
-  The resulting constant can be passed to functions accepting a
-  QLatin1String, a QString, or a QVariant.
-
-  The first parameter is the name of the variable to declare.  The
-  second parameter is the value of the constant, as a string literal.
-
-  For example:
-  \code
-  // in a header file
-  Q_DECLARE_LATIN1_CONSTANT(MyConstant, "MYCONSTANT");
-  \endcode
-
-  The declaration should be paired with a matching Q_DEFINE_LATIN1_CONSTANT
-  with the same arguments to actually define the constant.
-
-  \sa Q_DEFINE_LATIN1_CONSTANT
-*/
-
-/*!
-  \macro Q_DEFINE_LATIN1_CONSTANT
-  \relates QLatin1Constant
-
-  This macro, along with the related Q_DECLARE_LATIN1_CONSTANT macro,
-  allows you to describe a "Latin 1 string constant".
-
-  The resulting constant can be passed to functions accepting a
-  QLatin1String, a QString, or a QVariant.
-
-  The first parameter is the name of the variable to define.  The
-  second parameter is the value of the constant, as a string literal.
-
-  For example:
-  \code
-  // in a header file
-  Q_DECLARE_LATIN1_CONSTANT(MyConstant, "MYCONSTANT");
-
-  // in source file
-  Q_DEFINE_LATIN1_CONSTANT(MyConstant, "MYCONSTANT");
-  \endcode
-
-  You can use this macro without the matching DECLARE macro if
-  you are using the constant only in a single compilation unit.
-
-  \sa Q_DECLARE_LATIN1_CONSTANT
-*/
-
 /*!
   \fn QContactDetail::operator!=(const QContactDetail& other) const
   Returns true if the values or id of this detail is different to those of the \a other detail
@@ -486,6 +391,49 @@ QDebug operator<<(QDebug dbg, const QContactDetail& detail)
 }
 #endif
 
+#ifndef QT_NO_DATASTREAM
+/*!
+ * Writes \a detail to the stream \a out.
+ */
+QDataStream& operator<<(QDataStream& out, const QContactDetail& detail)
+{
+    quint8 formatVersion = 1; // Version of QDataStream format for QContactDetail
+    return out << formatVersion
+               << detail.definitionName()
+               << static_cast<quint32>(detail.accessConstraints())
+               << detail.variantValues();
+}
+
+/*!
+ * Reads a contact detail from stream \a in into \a detail.
+ */
+QDataStream& operator>>(QDataStream& in, QContactDetail& detail)
+{
+    detail = QContactDetail();
+    quint8 formatVersion;
+    in >> formatVersion;
+    if (formatVersion == 1) {
+        QString definitionName;
+        quint32 accessConstraintsInt;
+        QVariantMap values;
+        in >> definitionName >> accessConstraintsInt >> values;
+
+        detail = QContactDetail(definitionName);
+        QContactDetail::AccessConstraints accessConstraints(accessConstraintsInt);
+        detail.d->m_access = accessConstraints;
+
+        QMapIterator<QString, QVariant> it(values);
+        while (it.hasNext()) {
+            it.next();
+            detail.setValue(it.key(), it.value());
+        }
+    } else {
+        in.setStatus(QDataStream::ReadCorruptData);
+    }
+    return in;
+}
+#endif
+
 /*!
     Returns true if no values are contained in this detail.  Note that context is stored as a value; hence, if a context is set, this function will return false.
  */
@@ -496,7 +444,16 @@ bool QContactDetail::isEmpty() const
     return true;
 }
 
-/*! Returns the key of this detail. */
+/*!
+ * Returns the key of this detail.
+ *
+ * Be aware that if a contact is retrieved (or reloaded) from the backend, the
+ * keys of any details it contains may have been changed by the backend, or other
+ * threads may have modified the contact details in the backend.  Therefore,
+ * clients should reload the detail that they wish to save in or remove from a contact
+ * after retrieving the contact from the backend, in order to ascertain the keys of
+ * any such details.
+ */
 int QContactDetail::key() const
 {
     return d->m_id;
