@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -47,28 +41,31 @@
 #ifndef QMOBILITYGLOBAL_H
 #define QMOBILITYGLOBAL_H
 
-
-#define QTM_VERSION_STR "1.0.2"
+#define QTM_VERSION_STR "1.1.0"
 /*
    QTM_VERSION is (major << 16) + (minor << 8) + patch.
 */
-#define QTM_VERSION 0x010002
+#define QTM_VERSION 0x010100
 /*
-   can be used like #if (QTM_VERSION >= QTM_VERSION_CHECK(1, 0, 2))
+   can be used like #if (QTM_VERSION >= QTM_VERSION_CHECK(1, 1, 0))
 */
 #define QTM_VERSION_CHECK(major, minor, patch) ((major<<16)|(minor<<8)|(patch))
 
-#define QTM_PACKAGEDATE_STR "2010-07-27"
+#define QTM_PACKAGEDATE_STR "2010-11-01"
 
-#define QTM_PACKAGE_TAG "28039bded87c91474dc1252877b5b7d3f1d0ad1b"
+#define QTM_PACKAGE_TAG "fbbf8dad1046695fa14713f08ecdd19b81c67ea6"
 
-#if defined(QTM_BUILD_UNITTESTS)
-# include <qconfig.h>
-# if !defined(QT_BUILD_INTERNAL)
-#   define QT_BUILD_INTERNAL
-# endif
-#endif
 #include <QtCore/qglobal.h>
+#if defined(QTM_BUILD_UNITTESTS) && (defined(Q_OS_WIN) || defined(Q_OS_SYMBIAN)) && defined(QT_MAKEDLL)
+#    define QM_AUTOTEST_EXPORT Q_DECL_EXPORT
+#elif defined(QTM_BUILD_UNITTESTS) && (defined(Q_OS_WIN) || defined(Q_OS_SYMBIAN)) && defined(QT_DLL)
+#    define QM_AUTOTEST_EXPORT Q_DECL_IMPORT
+#elif defined(QTM_BUILD_UNITTESTS) && !(defined(Q_OS_WIN) || defined(Q_OS_SYMBIAN)) && defined(QT_SHARED)
+#    define QM_AUTOTEST_EXPORT Q_DECL_EXPORT
+#else
+#    define QM_AUTOTEST_EXPORT
+#endif
+
 
 #if defined(Q_OS_WIN) || defined(Q_OS_SYMBIAN)
 #  if defined(QT_NODLL)
@@ -98,6 +95,11 @@
 #    else
 #      define Q_VERSIT_EXPORT Q_DECL_IMPORT
 #    endif
+#    if defined(QT_BUILD_VERSIT_ORGANIZER_LIB)
+#      define Q_VERSIT_ORGANIZER_EXPORT Q_DECL_EXPORT
+#    else
+#      define Q_VERSIT_ORGANIZER_EXPORT Q_DECL_IMPORT
+#    endif
 #    if defined(QT_BUILD_LOCATION_LIB)
 #      define Q_LOCATION_EXPORT Q_DECL_EXPORT
 #    else
@@ -108,7 +110,7 @@
 #    else
 #      define Q_MESSAGING_EXPORT Q_DECL_IMPORT
 #    endif
-#      if defined(QT_BUILD_MULTIMEDIA_LIB)
+#    if defined(QT_BUILD_MULTIMEDIA_LIB)
 #        define Q_MULTIMEDIA_EXPORT Q_DECL_EXPORT
 #    else
 #        define Q_MULTIMEDIA_EXPORT Q_DECL_IMPORT
@@ -128,13 +130,29 @@
 #    else
 #      define Q_SENSORS_EXPORT Q_DECL_IMPORT
 #    endif
+#    if defined(QT_BUILD_FEEDBACK_LIB)
+#      define Q_FEEDBACK_EXPORT Q_DECL_EXPORT
+#    else
+#      define Q_FEEDBACK_EXPORT Q_DECL_IMPORT
+#    endif
+#    if defined(QT_BUILD_GALLERY_LIB)
+#      define Q_GALLERY_EXPORT Q_DECL_EXPORT
+#    else
+#      define Q_GALLERY_EXPORT Q_DECL_IMPORT
+#    endif
+#    if defined(QT_BUILD_ORGANIZER_LIB)
+#      define Q_ORGANIZER_EXPORT Q_DECL_EXPORT
+#    else
+#      define Q_ORGANIZER_EXPORT Q_DECL_IMPORT
+#    endif
 #  elif defined(QT_DLL) /* use a Qt DLL library */
 #    define Q_BEARER_EXPORT Q_DECL_IMPORT
 #    define Q_PUBLISHSUBSCRIBE_EXPORT Q_DECL_IMPORT
 #    define Q_CONTACTS_EXPORT Q_DECL_IMPORT
 #    define Q_VERSIT_EXPORT Q_DECL_IMPORT
+#    define Q_VERSIT_ORGANIZER_EXPORT Q_DECL_IMPORT
 #    define Q_LOCATION_EXPORT Q_DECL_IMPORT
-#      define Q_MULTIMEDIA_EXPORT Q_DECL_IMPORT
+#    define Q_MULTIMEDIA_EXPORT Q_DECL_IMPORT
 #    define Q_MESSAGING_EXPORT Q_DECL_IMPORT
 #    if QTM_SERVICEFW_SYMBIAN_DATABASEMANAGER_SERVER
 #      define Q_SERVICEFW_EXPORT
@@ -143,6 +161,9 @@
 #    endif
 #    define Q_SYSINFO_EXPORT Q_DECL_IMPORT
 #    define Q_SENSORS_EXPORT Q_DECL_IMPORT
+#    define Q_FEEDBACK_EXPORT Q_DECL_IMPORT
+#    define Q_GALLERY_EXPORT Q_DECL_IMPORT
+#    define Q_ORGANIZER_EXPORT Q_DECL_IMPORT
 #  endif
 #endif
 
@@ -152,23 +173,31 @@
 #    define Q_PUBLISHSUBSCRIBE_EXPORT Q_DECL_EXPORT
 #    define Q_CONTACTS_EXPORT Q_DECL_EXPORT
 #    define Q_VERSIT_EXPORT Q_DECL_EXPORT
+#    define Q_VERSIT_ORGANIZER_EXPORT Q_DECL_EXPORT
 #    define Q_LOCATION_EXPORT Q_DECL_EXPORT
 #    define Q_MULTIMEDIA_EXPORT Q_DECL_EXPORT
 #    define Q_MESSAGING_EXPORT Q_DECL_EXPORT
 #    define Q_SERVICEFW_EXPORT Q_DECL_EXPORT
 #    define Q_SYSINFO_EXPORT Q_DECL_EXPORT
 #    define Q_SENSORS_EXPORT Q_DECL_EXPORT
+#    define Q_FEEDBACK_EXPORT Q_DECL_EXPORT
+#    define Q_GALLERY_EXPORT Q_DECL_EXPORT
+#    define Q_ORGANIZER_EXPORT Q_DECL_EXPORT
 #  else
 #    define Q_BEARER_EXPORT
 #    define Q_PUBLISHSUBSCRIBE_EXPORT
 #    define Q_CONTACTS_EXPORT
 #    define Q_VERSIT_EXPORT
+#    define Q_VERSIT_ORGANIZER_EXPORT
 #    define Q_LOCATION_EXPORT
 #    define Q_MULTIMEDIA_EXPORT
 #    define Q_MESSAGING_EXPORT
 #    define Q_SERVICEFW_EXPORT
 #    define Q_SYSINFO_EXPORT
 #    define Q_SENSORS_EXPORT
+#    define Q_FEEDBACK_EXPORT
+#    define Q_GALLERY_EXPORT
+#    define Q_ORGANIZER_EXPORT
 #  endif
 #endif
 
@@ -178,10 +207,10 @@
 #    undef Q_SERVICEFW_EXPORT
 #  endif
 #  define Q_SERVICEFW_EXPORT
-#  ifdef Q_AUTOTEST_EXPORT
-#    undef Q_AUTOTEST_EXPORT
+#  ifdef QM_AUTOTEST_EXPORT
+#    undef QM_AUTOTEST_EXPORT
 #  endif
-#  define Q_AUTOTEST_EXPORT
+#  define QM_AUTOTEST_EXPORT
 #endif
 
 // The namespace is hardcoded as moc has issues resolving
@@ -204,3 +233,4 @@
 QT_USE_NAMESPACE
 
 #endif // QMOBILITYGLOBAL_H
+
