@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -48,6 +42,8 @@
 
 #if defined(Q_OS_SYMBIAN)
 #   include "qgeopositioninfosource_s60_p.h"
+#elif defined(QT_SIMULATOR)
+#   include "qgeopositioninfosource_simulator_p.h"
 #elif defined(Q_OS_WINCE)
 #   include "qgeopositioninfosource_wince_p.h"
 #elif defined(Q_WS_MAEMO_6)
@@ -61,6 +57,9 @@ QTM_BEGIN_NAMESPACE
 /*!
     \class QGeoPositionInfoSource
     \brief The QGeoPositionInfoSource class is an abstract base class for the distribution of positional updates.
+
+    \inmodule QtLocation
+    
     \ingroup location
 
     The static function QGeoPositionInfoSource::createDefaultSource() creates a default
@@ -145,9 +144,9 @@ QGeoPositionInfoSource::~QGeoPositionInfoSource()
     interval is less than the minimumUpdateInterval(),
     the minimum interval is used instead.
 
-    Changes to the update interval will happen as soon as is practical, however the 
-    time the change takes may vary between implementations.  Whether or not the elapsed 
-    time from the previous interval is counted as part of the new interval is also 
+    Changes to the update interval will happen as soon as is practical, however the
+    time the change takes may vary between implementations.  Whether or not the elapsed
+    time from the previous interval is counted as part of the new interval is also
     implementation dependent.
 
     The default value for this property is 0.
@@ -210,6 +209,8 @@ QGeoPositionInfoSource *QGeoPositionInfoSource::createDefaultSource(QObject *par
     if (error != KErrNone)
         return 0;
     return ret;
+#elif defined(QT_SIMULATOR)
+    return new QGeoPositionInfoSourceSimulator(parent);
 #elif defined(Q_OS_WINCE)
     return new QGeoPositionInfoSourceWinCE(parent);
 #elif (defined(Q_WS_MAEMO_6)) || (defined(Q_WS_MAEMO_5))
