@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -61,7 +55,9 @@
 #include "qmessagefilter_p.h"
 #include "qmessagefolderfilter.h"
 #include "qmessageservice.h"
-
+#ifdef QTHIGHWAYUSED
+#include <xqappmgr.h>
+#endif
 
 class CRichText;
 class CCharFormatLayer;
@@ -91,6 +87,12 @@ struct MessageEvent
     bool unfiltered;
 };
 
+struct PendingSend
+{
+    TMsvId messageId;
+    QMessageServicePrivate* privateService;
+};
+
 struct MessageQueryInfo
 {
     int operationId;
@@ -106,10 +108,13 @@ struct MessageQueryInfo
     int currentFilterListIndex;
     QMessageIdList ids;
     int count;
+    bool canceled;
 };
 
-class CMTMEngine : public CActive, public MMsvSessionObserver
+class CMTMEngine : public QObject, public CActive, public MMsvSessionObserver
 {
+    Q_OBJECT
+
 public:
     enum MTMType
     {
@@ -147,11 +152,11 @@ public:
     QMessage message(const QMessageId& id) const;
     
     bool storeMMS(QMessage &message);
-    bool sendMMS(QMessage &message);
+    bool sendMMS(QMessageServicePrivate& privateService, QMessage &message);
     bool storeEmail(QMessage &message);
-    bool sendEmail(QMessage &message);
+    bool sendEmail(QMessageServicePrivate& privateService, QMessage &message);
     bool storeSMS(QMessage &message);
-    bool sendSMS(QMessage &message);
+    bool sendSMS(QMessageServicePrivate& privateService, QMessage &message);
     bool retrieve(QMessageServicePrivate& privateService, const QMessageId &messageId, const QMessageContentContainerId& id);
     bool retrieveBody(QMessageServicePrivate& privateService, const QMessageId& id);
     bool retrieveHeader(QMessageServicePrivate& privateService, const QMessageId& id);
@@ -167,9 +172,14 @@ public:
     void notification(TMsvSessionEvent aEvent, TUid aMsgType, TMsvId aFolderId, TMsvId aMessageId);
     void filterAndOrderMessagesReady(bool success, int operationId, QMessageIdList ids, int numberOfHandledFilters,
                                      bool resultSetOrdered);
+    
+    void cancel(QMessageServicePrivate& privateService);
 
     inline RFs& FsSession() const { return((RFs&)iFsSession); }
 
+public slots:
+    void cleanupMTMBackend();
+    
 private:
     void updateEmailAccountsL() const;
     bool switchToMTMRootEntry(MTMType aMTMType);
@@ -229,12 +239,12 @@ private:
     void showMessageL(const QMessageId &id);
     
     void storeMMSL(QMessage &message);
-    void sendMMSL(QMessage &message);
+    void sendMMSL(QMessageServicePrivate& privateService, QMessage &message);
     void storeEmailL(QMessage &message);
-    void sendEmailL(QMessage &message);
+    void sendEmailL(QMessageServicePrivate& privateService, QMessage &message);
     void storeSMSL(QMessage &message);
     bool validateSMS();
-    void sendSMSL(QMessage &message);
+    void sendSMSL(QMessageServicePrivate& privateService, QMessage &message);
     void retrieveL(QMessageServicePrivate& privateService, const QMessageId &messageId, const QMessageContentContainerId& id);
     void retrieveBodyL(QMessageServicePrivate& privateService, const QMessageId& id);
     void retrieveHeaderL(QMessageServicePrivate& privateService, const QMessageId& id);
@@ -255,6 +265,9 @@ private:
                                                               CBaseMtm* mtm,
                                                               TMsvId serviceId);
     void deleteAsynchronousMTMOperation(CAsynchronousMTMOperation *apOperation);
+    bool checkIfWaitingDiscardClearMessage(TMsvId aMessageId);
+    
+    bool isGsm0338CompatibleUnicodeCharacter(TUint16 aValue);
 
 private: // from CActive
     void RunL();
@@ -304,14 +317,21 @@ private:
     QList<MessageEvent> iUndeliveredMessageEvents;
     RTimer              iTimer; // Timer used for delaying delivering of received
                                 // messages until messages are ready to be read
+    TMsvId iMessageId;
+    bool iNewMessage;
     
     mutable int iOperationIds;
     mutable QList<MessageQueryInfo> iMessageQueries;
+    mutable QList<PendingSend> iPendingSends;
     
     mutable QMessageAccountSortOrder iCurrentAccountOrdering;
     mutable QMessageFolderSortOrder iCurrentFolderOrdering;
     mutable QMessageSortOrder iCurrentMessageOrdering;
     
+#ifdef QTHIGHWAYUSED    
+    XQApplicationManager iAiwMgr;
+#endif
+    
     friend class QMessageService;
     friend class CMessagesFindOperation;
     friend class CAsynchronousMTMOperation;
