@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -53,7 +47,7 @@
 // of other Qt classes.  This header file may change from version to
 // version without notice, or even be removed.
 //
-// We mean it.
+// INTERNAL USE ONLY: Do NOT use for any other purpose.
 //
 
 #include <QtCore/qcoreapplication.h>
@@ -162,7 +156,7 @@ int QAudioInputPrivate::xrun_recovery(int err)
 
 int QAudioInputPrivate::setFormat()
 {
-    snd_pcm_format_t format = SND_PCM_FORMAT_S16;
+    snd_pcm_format_t format = SND_PCM_FORMAT_UNKNOWN;
 
     if(settings.sampleSize() == 8) {
         format = SND_PCM_FORMAT_U8;
@@ -214,7 +208,9 @@ int QAudioInputPrivate::setFormat()
             format = SND_PCM_FORMAT_FLOAT64_BE;
     }
 
-    return snd_pcm_hw_params_set_format( handle, hwparams, format);
+    return format != SND_PCM_FORMAT_UNKNOWN
+            ? snd_pcm_hw_params_set_format( handle, hwparams, format)
+            : -1;
 }
 
 void QAudioInputPrivate::start(QIODevice* device)
@@ -280,10 +276,27 @@ bool QAudioInputPrivate::open()
     elapsedTimeOffset = 0;
 
     int dir;
-    int err=-1;
+    int err = 0;
     int count=0;
     unsigned int freakuency=settings.frequency();
 
+    if (!settings.isValid()) {
+        qWarning("QAudioOutput: open error, invalid format.");
+    } else if (settings.sampleRate() <= 0) {
+        qWarning("QAudioOutput: open error, invalid sample rate (%d).",
+                 settings.sampleRate());
+    } else {
+        err = -1;
+    }
+
+    if (err == 0) {
+        errorState = QAudio::OpenError;
+        deviceState = QAudio::StoppedState;
+        emit errorChanged(errorState);
+        return false;
+    }
+
+
     QString dev = QString(QLatin1String(m_device.constData()));
     QList<QByteArray> devices = QAudioDeviceInfoInternal::availableDevices(QAudio::AudioInput);
     if(dev.compare(QLatin1String("default")) == 0) {
@@ -747,3 +760,5 @@ void InputPrivate::trigger()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qaudioinput_alsa_p.cpp"
