@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -53,7 +47,7 @@
 // of other Qt classes.  This header file may change from version to
 // version without notice, or even be removed.
 //
-// We mean it.
+// INTERNAL USE ONLY: Do NOT use for any other purpose.
 //
 
 #include <CoreServices/CoreServices.h>
@@ -66,11 +60,11 @@
 #include <QtCore/qtimer.h>
 #include <QtCore/qdebug.h>
 
-#include <qaudiodeviceinfo.h>
 #include <qaudiooutput.h>
 
 #include "qaudio_mac_p.h"
 #include "qaudiooutput_mac_p.h"
+#include "qaudiodeviceinfo_mac_p.h"
 
 
 QT_BEGIN_NAMESPACE
@@ -283,6 +277,7 @@ QAudioOutputPrivate::QAudioOutputPrivate(const QByteArray& device)
     if (QAudio::Mode(mode) == QAudio::AudioInput)
         errorCode = QAudio::OpenError;
     else {
+        audioDeviceInfo = new QAudioDeviceInfoInternal(device, QAudio::AudioOutput);
         isOpen = false;
         audioDeviceId = AudioDeviceID(did);
         audioUnit = 0;
@@ -304,6 +299,7 @@ QAudioOutputPrivate::QAudioOutputPrivate(const QByteArray& device)
 
 QAudioOutputPrivate::~QAudioOutputPrivate()
 {
+    delete audioDeviceInfo;
     close();
 }
 
@@ -363,17 +359,7 @@ bool QAudioOutputPrivate::open()
     // Set stream format
     streamFormat = toAudioStreamBasicDescription(audioFormat);
 
-    UInt32 size = sizeof(deviceFormat);
-    if (AudioUnitGetProperty(audioUnit,
-                                kAudioUnitProperty_StreamFormat,
-                                kAudioUnitScope_Input,
-                                0,
-                                &deviceFormat,
-                                &size) != noErr) {
-        qWarning() << "QAudioOutput: Unable to retrieve device format";
-        return false;
-    }
-
+    UInt32 size = sizeof(streamFormat);
     if (AudioUnitSetProperty(audioUnit,
                                 kAudioUnitProperty_StreamFormat,
                                 kAudioUnitScope_Input,
@@ -397,8 +383,7 @@ bool QAudioOutputPrivate::open()
         return false;
     }
 
-    periodSizeBytes = (numberOfFrames * streamFormat.mSampleRate / deviceFormat.mSampleRate) * 
-                        streamFormat.mBytesPerFrame;
+    periodSizeBytes = numberOfFrames * streamFormat.mBytesPerFrame;
     if (internalBufferSize < periodSizeBytes * 2)
         internalBufferSize = periodSizeBytes * 2;
     else
@@ -446,7 +431,7 @@ void QAudioOutputPrivate::start(QIODevice* device)
 {
     QIODevice*  op = device;
 
-    if (!audioFormat.isValid() || !open()) {
+    if (!audioDeviceInfo->isFormatSupported(audioFormat) || !open()) {
         stateCode = QAudio::StoppedState;
         errorCode = QAudio::OpenError;
     }
@@ -477,7 +462,7 @@ QIODevice* QAudioOutputPrivate::start()
 {
     QIODevice*  op = 0;
 
-    if (!audioFormat.isValid() || !open()) {
+    if (!audioDeviceInfo->isFormatSupported(audioFormat) || !open()) {
         stateCode = QAudio::StoppedState;
         errorCode = QAudio::OpenError;
         return audioIO;
@@ -751,4 +736,3 @@ OSStatus QAudioOutputPrivate::renderCallback(void* inRefCon,
 QT_END_NAMESPACE
 
 #include "qaudiooutput_mac_p.moc"
-
