@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -50,14 +44,17 @@
 #include "qmediaobject_p.h"
 #include "qmediaimageviewerservice_p.h"
 
+#include <qgraphicsvideoitem.h>
 #include <qmediaplaylist.h>
 #include <qmediaplaylistsourcecontrol.h>
 #include <qmediacontent.h>
 #include <qmediaresource.h>
+#include <qvideowidget.h>
 
 #include <QtCore/qcoreevent.h>
-#include <QtCore/qtextstream.h>
 #include <QtCore/qdatetime.h>
+#include <QtCore/qpointer.h>
+#include <QtCore/qtextstream.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -77,6 +74,7 @@ public:
 
     QMediaImageViewerControl *viewerControl;
     QMediaPlaylist *playlist;
+    QPointer<QObject> videoOutput;
     QMediaImageViewer::State state;
     int timeout;
     int pauseTime;
@@ -137,19 +135,19 @@ void QMediaImageViewerPrivate::_q_playlistDestroyed()
 /*!
     \class QMediaImageViewer
     \brief The QMediaImageViewer class provides a means of viewing image media.
+    \inmodule QtMultimediaKit
     \ingroup multimedia
     \preliminary
 
     QMediaImageViewer is used together with a media display object such as
     QVideoWidget to present an image.  A display object is attached to the
-    image viewer by passing a pointer to the QMediaImageViewer instance to the
-    setMediaObject() function of the display object.
+    image viewer by means of the bind function.
 
     \code
     viewer = new QMediaImageViewer(this);
 
     display = new QVideoWidget;
-    viewer->setVideoOutput(display);
+    viewer->bind(display);
     display->show();
     \endcode
 
@@ -166,12 +164,12 @@ void QMediaImageViewerPrivate::_q_playlistDestroyed()
 
     \code
     playlist = new QMediaPlaylist(this);
-    playlist->setMediaObject(viewer);
     playlist->setPlaybackMode(QMediaPlaylist::Loop);
     playlist->addMedia(image1);
     playlist->addMedia(image2);
     playlist->addMedia(image3);
 
+    viewer->setPlaylist(playlist);
     viewer->setTimeout(5000);
     viewer->play();
     \endcode
@@ -313,7 +311,7 @@ void QMediaImageViewer::setPlaylist(QMediaPlaylist *playlist)
                    this, SLOT(_q_playlistMediaChanged(QMediaContent)));
         disconnect(d->playlist, SIGNAL(destroyed()), this, SLOT(_q_playlistDestroyed()));
 
-        QMediaObject::unbind(playlist);
+        QMediaObject::unbind(d->playlist);
     }
 
     d->playlist = playlist;
@@ -401,6 +399,38 @@ int QMediaImageViewer::elapsedTime() const
     \sa timeout, QMediaObject::notifyInterval
 */
 
+/*!
+    Sets a video \a widget as the current video output.
+
+    This will unbind any previous video output bound with setVideoOutput().
+*/
+
+void QMediaImageViewer::setVideoOutput(QVideoWidget *widget)
+{
+    Q_D(QMediaImageViewer);
+
+    if (d->videoOutput)
+        unbind(d->videoOutput);
+
+    d->videoOutput = bind(widget) ? widget : 0;
+}
+
+/*!
+    Sets a video \a item as the current video output.
+
+    This will unbind any previous video output bound with setVideoOutput().
+*/
+
+void QMediaImageViewer::setVideoOutput(QGraphicsVideoItem *item)
+{
+    Q_D(QMediaImageViewer);
+
+    if (d->videoOutput)
+        unbind(d->videoOutput);
+
+    d->videoOutput = bind(item) ? item : 0;
+}
+
 /*!
     \internal
 */
