@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -55,6 +49,8 @@
 #include <qaudioencodercontrol.h>
 #include <qvideoencodercontrol.h>
 #include <qmediacontainercontrol.h>
+#include <qcamera.h>
+#include <qcameracontrol.h>
 
 #include <QtCore/qdebug.h>
 #include <QtCore/qurl.h>
@@ -67,6 +63,7 @@ QT_BEGIN_NAMESPACE
 
 /*!
     \class QMediaRecorder
+    \inmodule QtMultimediaKit
     \ingroup multimedia
 
     \preliminary
@@ -124,6 +121,8 @@ public:
     QVideoEncoderControl *videoControl;
     QMetaDataWriterControl *metaDataControl;
 
+    QTimer* notifyTimer;
+
     QMediaRecorder::State state;
     QMediaRecorder::Error error;
     QString errorString;
@@ -131,6 +130,8 @@ public:
     void _q_stateChanged(QMediaRecorder::State state);
     void _q_error(int error, const QString &errorString);
     void _q_serviceDestroyed();
+    void _q_notify();
+    void _q_updateNotifyInterval(int ms);
 
     QMediaRecorder *q_ptr;
 };
@@ -142,6 +143,7 @@ QMediaRecorderPrivate::QMediaRecorderPrivate():
      audioControl(0),
      videoControl(0),
      metaDataControl(0),
+     notifyTimer(0),
      state(QMediaRecorder::StoppedState),
      error(QMediaRecorder::NoError)
 {
@@ -153,12 +155,10 @@ void QMediaRecorderPrivate::_q_stateChanged(QMediaRecorder::State ps)
 {
     Q_Q(QMediaRecorder);
 
-    /*
     if (ps == QMediaRecorder::RecordingState)
-        q->addPropertyWatch("duration");
+        notifyTimer->start();
     else
-        q->removePropertyWatch("duration");
-    */
+        notifyTimer->stop();
 
 //    qDebug() << "Recorder state changed:" << ENUM_NAME(QMediaRecorder,"State",ps);
     if (state != ps) {
@@ -184,6 +184,16 @@ void QMediaRecorderPrivate::_q_serviceDestroyed()
     q_func()->setMediaObject(0);
 }
 
+void QMediaRecorderPrivate::_q_notify()
+{
+    emit q_func()->durationChanged(q_func()->duration());
+}
+
+void QMediaRecorderPrivate::_q_updateNotifyInterval(int ms)
+{
+    notifyTimer->setInterval(ms);
+}
+
 
 /*!
     Constructs a media recorder which records the media produced by \a mediaObject.
@@ -198,6 +208,11 @@ QMediaRecorder::QMediaRecorder(QMediaObject *mediaObject, QObject *parent):
     Q_D(QMediaRecorder);
     d->q_ptr = this;
     setMediaObject(mediaObject);
+
+    d->notifyTimer = new QTimer(this);
+    d->notifyTimer->setInterval(mediaObject->notifyInterval());
+    connect(d->notifyTimer, SIGNAL(timeout()), SLOT(_q_notify()));
+    connect(mediaObject, SIGNAL(notifyIntervalChanged(int)), SLOT(_q_updateNotifyInterval(int)));
 }
 
 /*!
@@ -208,11 +223,18 @@ QMediaRecorder::~QMediaRecorder()
 {
 }
 
+/*!
+    Returns the QMediaObject instance that this QMediaRecorder is bound too,
+    or 0 otherwise.
+*/
 QMediaObject *QMediaRecorder::mediaObject() const
 {
     return d_func()->mediaObject;
 }
 
+/*!
+    \internal
+*/
 bool QMediaRecorder::setMediaObject(QMediaObject *object)
 {
     Q_D(QMediaRecorder);
@@ -332,6 +354,10 @@ bool QMediaRecorder::setMediaObject(QMediaObject *object)
     local file system locations but a network URL was passed. If the service
     does not support media recording this setting the output location will
     always fail.
+
+    The \a location can be relative or empty;
+    in this case the recorder uses the system specific place and file naming scheme.
+    After recording has stated, QMediaRecorder::outputLocation() returns the actual output location.
 */
 
 /*!
@@ -606,6 +632,14 @@ void QMediaRecorder::setEncodingSettings(const QAudioEncoderSettings &audio,
 {
     Q_D(QMediaRecorder);
 
+    QCamera *camera = qobject_cast<QCamera*>(d->mediaObject);
+    if (camera && camera->captureMode() == QCamera::CaptureVideo) {
+        QMetaObject::invokeMethod(camera,
+                                  "_q_preparePropertyChange",
+                                  Qt::DirectConnection,
+                                  Q_ARG(int, QCameraControl::VideoEncodingSettings));
+    }
+
     if (d->audioControl)
         d->audioControl->setAudioSettings(audio);
 
@@ -696,6 +730,11 @@ void QMediaRecorder::stop()
     Signals that an \a error has occurred.
 */
 
+/*!
+    \fn QMediaRecorder::mutedChanged(bool muted)
+
+    Signals that the \a muted state has changed. If true the recording is being muted.
+*/
 
 /*!
     \property QMediaRecorder::metaDataAvailable
