@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -51,6 +45,7 @@
 #include "qsensormanager.h"
 #include <QDebug>
 #include <QMetaProperty>
+#include <QTimer>
 
 QTM_BEGIN_NAMESPACE
 
@@ -75,9 +70,65 @@ static int qoutputrangelist_id = qRegisterMetaType<QtMobility::qoutputrangelist>
 
 // =====================================================================
 
+/*!
+    \class qoutputrange
+    \brief The qoutputrange class holds the specifics of an output range.
+
+    The class is defined as a simple struct.
+
+    \code
+    struct qoutputrange
+    {
+        qreal maximum;
+        qreal minimum;
+        qreal accuracy;
+    };
+    \endcode
+
+    Each output range specifies a minimum and maximum value as well as an accuracy value.
+    The accuracy value represents the resolution of the sensor. It is the smallest change
+    the sensor can detect and is expressed using the same units as the minimum and maximum.
+
+    Sensors must often trade off range for accuracy. To allow the user to determine which of
+    these are more important the sensor may offer several output ranges. One output
+    range may have reduced minimum and maximum values and increased sensitivity. Another output
+    range may have higher minimum and maximum values with reduced sensitivity. Note that higher
+    sensitivities will be represented by smaller accuracy values.
+
+    An example of this tradeoff can be seen by examining the LIS302DL accelerometer. It has only
+    256 possible values to report with. These values are scaled so that they can represent either
+    -2G to +2G (with an accuracy value of 0.015G) or -8G to +8G (with an accuracy value of 0.06G).
+
+    \sa QSensor::outputRanges
+*/
+
+/*!
+    \variable qoutputrange::maximum
+
+    This is the maximum value for this output range.
+    The units are defined by the sensor.
+*/
+
+/*!
+    \variable qoutputrange::minimum
+
+    This is the minimum value for this output range.
+    The units are defined by the sensor.
+*/
+
+/*!
+    \variable qoutputrange::accuracy
+
+    The accuracy value represents the resolution of the sensor. It is the smallest change
+    the sensor can detect and is expressed using the same units as the minimum and maximum.
+*/
+
+// =====================================================================
+
 /*!
     \class QSensor
     \ingroup sensors_main
+    \inmodule QtSensors
 
     \brief The QSensor class represents a single hardware sensor.
 
@@ -216,12 +267,28 @@ bool QSensor::isBusy() const
     be used to grab a sensor when it becomes available.
 */
 
+/*!
+    \fn QSensor::activeChanged()
+
+    This signal is emitted when the active property changes.
+*/
+
 /*!
     \property QSensor::active
     \brief a value to indicate if the sensor is active.
 
     This is true if the sensor is active (returning values). This is false otherwise.
 */
+void QSensor::setActive(bool active)
+{
+    if (active == d->active)
+        return;
+
+    if (active)
+        QTimer::singleShot(0, this, SLOT(start())); // delay ensures all properties have been set if using QML
+    else
+        stop();
+}
 
 bool QSensor::isActive() const
 {
@@ -304,7 +371,30 @@ void QSensor::setDataRate(int rate)
     Start retrieving values from the sensor.
     Returns true if the sensor was started, false otherwise.
 
-    Note that the sensor may fail to start for several reasons.
+    The sensor may fail to start for several reasons.
+
+    Once an application has started a sensor it must wait until the sensor receives a
+    new value before it can query the sensor's values. This is due to how the sensor
+    receives values from the system. Sensors do not (in general) poll for new values,
+    rather new values are pushed to the sensors as they happen.
+
+    For example, this code will not work as intended.
+
+    \badcode
+    sensor->start();
+    sensor->reading()->x(); // no data available
+    \endcode
+
+    To work correctly, the code that accesses the reading should ensure the
+    readingChanged() signal has been emitted.
+
+    \code
+        connect(sensor, SIGNAL(readingChanged()), this, SLOT(checkReading()));
+        sensor->start();
+    }
+    void MyClass::checkReading() {
+        sensor->reading()->x();
+    \endcode
 
     \sa QSensor::busy
 */
@@ -319,6 +409,7 @@ bool QSensor::start()
     d->busy = false;
     // Backend will update the flags appropriately
     d->backend->start();
+    Q_EMIT activeChanged();
     return d->active;
 }
 
@@ -335,6 +426,7 @@ void QSensor::stop()
         return;
     d->active = false;
     d->backend->stop();
+    Q_EMIT activeChanged();
 }
 
 /*!
@@ -345,7 +437,10 @@ void QSensor::stop()
 
     Note that this will return 0 until a sensor backend is connected to a backend.
 
-    \sa isConnectedToBackend()
+    Also note that readings are not immediately available after start() is called.
+    Applications must wait for the readingChanged() signal to be emitted.
+
+    \sa isConnectedToBackend(), start()
 */
 
 QSensorReading *QSensor::reading() const
@@ -394,6 +489,11 @@ void QSensor::removeFilter(QSensorFilter *filter)
     \fn QSensor::readingChanged()
 
     This signal is emitted when the reading has changed.
+
+    Before this signal has been emitted for the first time, the sensor reading will
+    have uninitialized data.
+
+    \sa start()
 */
 
 /*!
@@ -403,7 +503,13 @@ void QSensor::removeFilter(QSensorFilter *filter)
     A sensor may have more than one output range. Typically this is done
     to give a greater measurement range at the cost of lowering accuracy.
 
-    \sa QSensor::outputRange
+    The qoutputrangelist type exists for the benefit of the meta-type system.
+    It is just a typedef.
+    \code
+    typedef qoutputrangelist QList<qoutputrange>;
+    \endcode
+
+    \sa QSensor::outputRange, qoutputrange
 */
 
 qoutputrangelist QSensor::outputRanges() const
@@ -478,6 +584,7 @@ int QSensor::error() const
 /*!
     \class QSensorFilter
     \ingroup sensors_main
+    \inmodule QtSensors
 
     \brief The QSensorFilter class provides an efficient
            callback facility for asynchronous notifications of
@@ -542,6 +649,7 @@ void QSensorFilter::setSensor(QSensor *sensor)
 /*!
     \class QSensorReading
     \ingroup sensors_main
+    \inmodule QtSensors
 
     \brief The QSensorReading class holds the readings from the sensor.
 
