@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -49,6 +43,9 @@
 #include "qserviceplugininterface.h"
 #include "qabstractsecuritysession.h"
 #include "qserviceinterfacedescriptor_p.h"
+#include "qremoteserviceregister_p.h"
+#include "qremoteserviceregisterentry_p.h"
+
 #ifdef Q_OS_SYMBIAN
     #include "databasemanager_symbian_p.h"
 #else
@@ -64,7 +61,6 @@
 
 QTM_BEGIN_NAMESPACE
 
-
 static QString qservicemanager_resolveLibraryPath(const QString &libNameOrPath)
 {
     if (QFile::exists(libNameOrPath))
@@ -77,7 +73,7 @@ static QString qservicemanager_resolveLibraryPath(const QString &libNameOrPath)
 #endif
     for (int i=0; i<paths.count(); i++) {
         QString libPath = QDir::toNativeSeparators(paths[i]) + QDir::separator() + libNameOrPath;
-        
+
 #ifdef Q_OS_SYMBIAN
         QFileInfo fi(libPath);
         if (fi.suffix() == QLatin1String("dll"))
@@ -90,18 +86,17 @@ static QString qservicemanager_resolveLibraryPath(const QString &libNameOrPath)
             lib.unload();
             return libPath;
         }
-#else       
+#else
         QLibrary lib(libPath);  
         if (lib.load()) {
             lib.unload();
             return lib.fileName();
         }
-#endif        
+#endif
     }
     return QString();
 }
 
-
 class QServicePluginCleanup : public QObject
 {
     Q_OBJECT
@@ -115,7 +110,7 @@ public:
     ~QServicePluginCleanup()
     {
         if (m_loader) {
-            m_loader->unload();
+            //m_loader->unload();
             delete m_loader;
         }
     }
@@ -240,7 +235,7 @@ private slots:
     \value ImplementationAlreadyExists Another service that implements the same interface version has previously been registered.
     \value PluginLoadingFailed The service plugin cannot be loaded.
     \value ComponentNotFound The service or interface implementation has not been registered.
-    \value ServiceCapabilityDenied The security session does not allow the service based on its capabilities.
+    \value ServiceCapabilityDenied The security session does not permit service access based on its capabilities.
     \value UnknownError An unknown error occurred.
 */
 
@@ -279,6 +274,7 @@ QServiceManager::QServiceManager(QObject *parent)
     : QObject(parent),
       d(new QServiceManagerPrivate(this))
 {
+    qRegisterMetaType<QService::UnrecoverableIPCError>("QService::UnrecoverableIPCError");
     d->scope = QService::UserScope;
 }
 
@@ -401,8 +397,27 @@ QObject* QServiceManager::loadInterface(const QServiceInterfaceDescriptor& descr
         return 0;
     }
 
-    QString serviceFilePath = qservicemanager_resolveLibraryPath(
-            descriptor.attribute(QServiceInterfaceDescriptor::Location).toString());
+    const QString location = descriptor.attribute(QServiceInterfaceDescriptor::Location).toString();
+    const bool isInterProcess = (descriptor.attribute(QServiceInterfaceDescriptor::ServiceType).toInt() 
+                                == QService::InterProcess);
+    if (isInterProcess) {
+        //ipc service
+        const QByteArray version = QString("%1.%2").arg(descriptor.majorVersion())
+                                                   .arg(descriptor.minorVersion()).toLatin1();
+
+        QRemoteServiceRegister::Entry serviceEntry;
+        serviceEntry.d->iface = descriptor.interfaceName();
+        serviceEntry.d->service = descriptor.serviceName();
+        serviceEntry.d->ifaceVersion = version;
+        QObject* service = QRemoteServiceRegisterPrivate::proxyForService(serviceEntry, location);
+        if (!service)
+            d->setError(InvalidServiceLocation);
+
+        //client owns proxy object
+        return service;
+    }
+
+    const QString serviceFilePath = qservicemanager_resolveLibraryPath(location);
     if (serviceFilePath.isEmpty()) {
         d->setError(InvalidServiceLocation);
         return 0;
@@ -447,7 +462,7 @@ QObject* QServiceManager::loadInterface(const QServiceInterfaceDescriptor& descr
         }
     }
 
-    loader->unload();
+    //loader->unload();
     delete loader;
     d->setError(PluginLoadingFailed);
 
@@ -465,7 +480,7 @@ QObject* QServiceManager::loadInterface(const QServiceInterfaceDescriptor& descr
     If \a interfaceName is not a known interface the returned pointer will be null.
 
     Note that using this function implies that service and client share
-    the implamentation of T which means that service and client become tightly coupled.
+    the implementation of T which means that service and client become tightly coupled.
     This may cause issue during later updates as certain changes may require code changes
     to the service and client.
 
@@ -490,7 +505,7 @@ QObject* QServiceManager::loadInterface(const QServiceInterfaceDescriptor& descr
     If the \a serviceDescriptor is not valid the returned pointer will be null.
 
     Note that using this function implies that service and client share
-    the implamentation of T which means that service and client become tightly coupled.
+    the implementation of T which means that service and client become tightly coupled.
     This may cause issue during later updates as certain changes may require code changes
     to the service and client.
 
@@ -560,8 +575,13 @@ bool QServiceManager::addService(QIODevice *device)
     DatabaseManager::DbScope scope = d->scope == QService::UserScope ?
             DatabaseManager::UserOnlyScope : DatabaseManager::SystemScope;
     ServiceMetaDataResults results = parser.parseResults();
-    bool result = d->dbManager->registerService(results, scope);
 
+    bool result = d->dbManager->registerService(results, scope);
+    
+    if (results.type == QService::InterProcess)
+        return result;
+
+    //test the new plug-in
     if (result) {
         QPluginLoader *loader = new QPluginLoader(qservicemanager_resolveLibraryPath(data.location));
         QServicePluginInterface *pluginIFace = qobject_cast<QServicePluginInterface *>(loader->instance());
@@ -572,7 +592,7 @@ bool QServiceManager::addService(QIODevice *device)
             result = false;
             d->dbManager->unregisterService(data.name, scope);
         }
-        loader->unload();
+        //loader->unload();
         delete loader;
     } else {
         d->setError();
@@ -612,8 +632,13 @@ bool QServiceManager::removeService(const QString& serviceName)
 
     QSet<QString> pluginPathsSet;
     QList<QServiceInterfaceDescriptor> descriptors = findInterfaces(serviceName);
-    for (int i=0; i<descriptors.count(); i++)
-        pluginPathsSet << descriptors[i].attribute(QServiceInterfaceDescriptor::Location).toString();
+    for (int i=0; i<descriptors.count(); i++) {
+        const QString loc = descriptors[i].attribute(QServiceInterfaceDescriptor::Location).toString();
+        const int type = descriptors[i].attribute(QServiceInterfaceDescriptor::ServiceType).toInt();
+        //exclude ipc services
+        if (type <= QService::Plugin)
+            pluginPathsSet << loc;
+    }
 
     QList<QString> pluginPaths = pluginPathsSet.toList();
     for (int i=0; i<pluginPaths.count(); i++) {
@@ -623,7 +648,7 @@ bool QServiceManager::removeService(const QString& serviceName)
             pluginIFace->uninstallService();
         else
             qWarning() << "QServiceManager: unable to invoke uninstallService() on removed service";
-        loader->unload();
+        //loader->unload();
         delete loader;
     }
 
