@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
@@ -79,6 +73,10 @@
 #define INTERFACE_DESCRIPTION_KEY "DESCRIPTION"
 #define SERVICE_INITIALIZED_KEY SERVICE_INITIALIZED_ATTR
 #define INTERFACE_CAPABILITY_KEY "CAPABILITIES"
+#define INTERFACE_SERVICETYPE_KEY "SERVICETYPE"
+
+//service prefixes
+#define SERVICE_IPC_PREFIX "_q_ipc_addr:"
 
 QTM_BEGIN_NAMESPACE
 
@@ -139,8 +137,14 @@ bool ServiceDatabase::open()
 
     path = m_databasePath;
     QFileInfo dbFileInfo(path);
-    if (!dbFileInfo.dir().exists()) {
-       if(!QDir::root().mkpath(dbFileInfo.path())) {
+    if (!dbFileInfo.dir().exists()) {        
+      // Create the path with QFile, avoids problems with S60 3.2/3.1
+      // Avoid security violation with PlatSec
+#ifndef Q_OS_SYMBIAN
+       QDir::root().mkpath(dbFileInfo.path());
+#endif
+       QFile file(path);
+       if(!file.open(QIODevice::ReadWrite)) {
            QString errorText("Could not create database directory: %1");
            m_lastError.setError(DBError::CannotCreateDbDir, errorText.arg(dbFileInfo.path()));
 #ifdef QT_SFW_SERVICEDATABASE_DEBUG
@@ -150,6 +154,7 @@ bool ServiceDatabase::open()
            close();
            return false;
         }
+        file.close();
     }
 
     m_connectionName = dbFileInfo.completeBaseName();
@@ -225,6 +230,12 @@ bool ServiceDatabase::open()
 //bool ServiceDatabase::registerService(ServiceMetaData &service)
 bool ServiceDatabase::registerService(const ServiceMetaDataResults &service, const QString &securityToken)
 {
+    // Derive the location name with the service type prefix to be stored
+    QString locationPrefix = service.location;
+    int type = service.interfaces[0].d->attributes[QServiceInterfaceDescriptor::ServiceType].toInt();
+    if (type == QService::InterProcess)
+        locationPrefix = SERVICE_IPC_PREFIX + service.location;
+
 #ifndef QT_SFW_SERVICEDATABASE_USE_SECURITY_TOKEN
     Q_UNUSED(securityToken);
 #else
@@ -262,7 +273,7 @@ bool ServiceDatabase::registerService(const ServiceMetaDataResults &service, con
     //See if the service's location has already been previously registered
     QString statement("SELECT Name from Service WHERE Location=? COLLATE NOCASE");
     QList<QVariant> bindValues;
-    bindValues.append(service.location);
+    bindValues.append(locationPrefix);
     if (!executeQuery(&query, statement, bindValues)) {
         rollbackTransaction(&query);
 #ifdef QT_SFW_SERVICEDATABASE_DEBUG
@@ -331,12 +342,14 @@ bool ServiceDatabase::registerService(const ServiceMetaDataResults &service, con
 
     // Checks done, create new rows into tables.
     statement = "INSERT INTO Service(ID,Name,Location) VALUES(?,?,?)";
+
+    qsrand(QTime::currentTime().msec());
     QString serviceID = QUuid::createUuid().toString();
 
     bindValues.clear();
     bindValues.append(serviceID);
     bindValues.append(service.name);
-    bindValues.append(service.location);
+    bindValues.append(locationPrefix);
 
     if (!executeQuery(&query, statement, bindValues)) {
         rollbackTransaction(&query);
@@ -722,6 +735,7 @@ bool ServiceDatabase::insertInterfaceData(QSqlQuery *query,const QServiceInterfa
         ++customIter;
     }
     m_lastError.setError(DBError::NoError);
+
     return true;
 }
 
@@ -896,13 +910,20 @@ QList<QServiceInterfaceDescriptor> ServiceDatabase::getInterfaces(const QService
         difference.clear();
         interface.d->customAttributes.clear();
         interface.d->attributes.clear();
-        interface.d->interfaceName =query.value(EBindIndex).toString();
+        interface.d->interfaceName = query.value(EBindIndex).toString();
         interface.d->serviceName = query.value(EBindIndex1).toString();
         interface.d->major = query.value(EBindIndex2).toInt();
         interface.d->minor = query.value(EBindIndex3).toInt();
 
-        interface.d->attributes[QServiceInterfaceDescriptor::Location]
-            = query.value(EBindIndex4).toString();
+        QString location = query.value(EBindIndex4).toString();
+        if (location.startsWith(SERVICE_IPC_PREFIX)) {
+            interface.d->attributes[QServiceInterfaceDescriptor::ServiceType] = QService::InterProcess;
+            interface.d->attributes[QServiceInterfaceDescriptor::Location] 
+                = location.remove(0,QString(SERVICE_IPC_PREFIX).size());
+        } else { 
+            interface.d->attributes[QServiceInterfaceDescriptor::ServiceType] = QService::Plugin;
+            interface.d->attributes[QServiceInterfaceDescriptor::Location] = location;
+        }
 
         serviceID = query.value(EBindIndex5).toString();
         if (!populateServiceProperties(&interface, serviceID)) {
@@ -1018,12 +1039,20 @@ QServiceInterfaceDescriptor ServiceDatabase::getInterface(const QString &interfa
     }
 
     interface.d = new QServiceInterfaceDescriptorPrivate;
-    interface.d->interfaceName = query.value(EBindIndex).toString();
+    interface.d->interfaceName =query.value(EBindIndex).toString();
     interface.d->serviceName = query.value(EBindIndex1).toString();
     interface.d->major = query.value(EBindIndex2).toInt();
     interface.d->minor = query.value(EBindIndex3).toInt();
-    interface.d->attributes[QServiceInterfaceDescriptor::Location]
-        = query.value(EBindIndex4).toString();
+
+    QString location = query.value(EBindIndex4).toString();
+    if (location.startsWith(SERVICE_IPC_PREFIX)) {
+        interface.d->attributes[QServiceInterfaceDescriptor::ServiceType] = QService::InterProcess;
+        interface.d->attributes[QServiceInterfaceDescriptor::Location] 
+            = location.remove(0,QString(SERVICE_IPC_PREFIX).size());
+    } else { 
+        interface.d->attributes[QServiceInterfaceDescriptor::ServiceType] = QService::Plugin;
+        interface.d->attributes[QServiceInterfaceDescriptor::Location] = location;
+    }
 
     QString serviceID = query.value(EBindIndex5).toString();
     if (!populateServiceProperties(&interface, serviceID)) {
@@ -1212,8 +1241,15 @@ QServiceInterfaceDescriptor ServiceDatabase::interfaceDefault(const QString &int
     interface.d->major = query.value(EBindIndex2).toInt();
     interface.d->minor = query.value(EBindIndex3).toInt();
 
-    interface.d->attributes[QServiceInterfaceDescriptor::Location]
-        = query.value(EBindIndex4).toString();
+    QString location = query.value(EBindIndex4).toString();
+    if (location.startsWith(SERVICE_IPC_PREFIX)) {
+        interface.d->attributes[QServiceInterfaceDescriptor::ServiceType] = QService::InterProcess;
+        interface.d->attributes[QServiceInterfaceDescriptor::Location] 
+            = location.remove(0,QString(SERVICE_IPC_PREFIX).size());
+    } else { 
+        interface.d->attributes[QServiceInterfaceDescriptor::ServiceType] = QService::Plugin;
+        interface.d->attributes[QServiceInterfaceDescriptor::Location] = location;
+    }
 
     QString serviceID = query.value(EBindIndex5).toString();
     if (!populateServiceProperties(&interface, serviceID)) {
