@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,20 +33,13 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
-#include "qsysteminfo.h"
-#include "qsysteminfo_maemo_p.h"
-
+#include "qsysteminfocommon_p.h"
+#include <qsysteminfo_maemo_p.h>
 #include <QStringList>
 #include <QSize>
 #include <QFile>
@@ -106,7 +99,11 @@ QStringList QSystemInfoPrivate::availableLanguages() const
 {
     QStringList languages;
 
+#if !defined(Q_WS_MAEMO_6)
     GConfItem languagesItem("/apps/osso/inputmethod/available_languages");
+#else
+    GConfItem languagesItem("/meegotouch/inputmethods/languages");
+#endif
     const QStringList locales = languagesItem.value().toStringList();
 
     foreach(const QString locale, locales) {
@@ -118,6 +115,27 @@ QStringList QSystemInfoPrivate::availableLanguages() const
     return languages;
 }
 
+QString QSystemInfoPrivate::currentLanguage() const
+{
+#if defined(Q_WS_MAEMO_6)
+    GConfItem langItem("/meegotouch/i18n/language");
+    return langItem.value().toString();
+#else
+    return QSystemInfoLinuxCommonPrivate::currentLanguage();
+#endif
+}
+
+
+QString QSystemInfoPrivate::currentCountryCode() const
+{
+#if defined(Q_WS_MAEMO_6)
+    GConfItem langItem("/meegotouch/i18n/language");
+    return langItem.value().toString().section("_",1,1);
+#else
+    return QSystemInfoLinuxCommonPrivate::currentCountryCode();
+#endif
+}
+
 QString QSystemInfoPrivate::version(QSystemInfo::Version type,
                                     const QString &parameter)
 {
@@ -130,15 +148,11 @@ QString QSystemInfoPrivate::version(QSystemInfo::Version type,
                                                "/com/nokia/SystemInfo",
                                                "com.nokia.SystemInfo",
                                                QDBusConnection::systemBus());
-            if(!connectionInterface.isValid()) {
-                qWarning() << "interfacenot valid";
-            } else {
-                QDBusReply< QByteArray > reply =
-                    connectionInterface.call("GetConfigValue",
-                                             "/device/sw-release-ver");
-                if(reply.isValid())
-                    return reply.value();
-            }
+            QDBusReply< QByteArray > reply =
+                connectionInterface.call("GetConfigValue",
+                                         "/device/sw-release-ver");
+            if(reply.isValid())
+                return reply.value();
             break;
         }
         default:
@@ -160,11 +174,21 @@ bool QSystemInfoPrivate::hasFeatureSupported(QSystemInfo::Feature feature)
         break;
     case QSystemInfo::LocationFeature :
         {
+#if defined(Q_WS_MAEMO_6)
+            GConfItem satellitePositioning("/system/osso/location/settings/satellitePositioning");
+            GConfItem networkPositioning("/system/osso/location/settings/networkPositioning");
+
+            bool satellitePositioningAvailable = satellitePositioning.value(false).toBool();
+            bool networkPositioningAvailable   = networkPositioning.value(false).toBool();
+
+            featureSupported = (satellitePositioningAvailable || networkPositioningAvailable);
+#else /* Maemo 5 */
             GConfItem locationValues("/system/nokia/location");
             const QStringList locationKeys = locationValues.listEntries();
             if(locationKeys.count()) {
                 featureSupported = true;
             }
+#endif /* Maemo 5 */
         }
         break;
     case QSystemInfo::HapticsFeature:
@@ -191,7 +215,19 @@ bool QSystemInfoPrivate::hasFeatureSupported(QSystemInfo::Feature feature)
 QSystemNetworkInfoPrivate::QSystemNetworkInfoPrivate(QSystemNetworkInfoLinuxCommonPrivate *parent)
         : QSystemNetworkInfoLinuxCommonPrivate(parent)
 {
-    setupNetworkInfo();
+    csStatusMaemo6["Unknown"]    = -1;  // Current registration status is unknown.
+    csStatusMaemo6["Home"]       = 0;   // Registered with the home network.
+    csStatusMaemo6["Roaming"]    = 1;   // Registered with a roaming network.
+    csStatusMaemo6["Offline"]    = 3;   // Not registered.
+    csStatusMaemo6["Searching"]  = 4;   // Offline, but currently searching for network.
+    csStatusMaemo6["NoSim"]      = 6;   // Offline because no SIM is present.
+    csStatusMaemo6["PowerOff"]   = 8;   // Offline because the CS is powered off.
+    csStatusMaemo6["PowerSave"]  = 9;   // Offline and in power save mode.
+    csStatusMaemo6["NoCoverage"] = 10;  // Offline and in power save mode because of poor coverage.
+    csStatusMaemo6["Rejected"]   = 11;  // Offline because SIM was rejected by the network.
+
+    QTimer::singleShot(0,this,SLOT(setupNetworkInfo()));
+    QCoreApplication::processEvents();//this needs to be called otherwise this timer never fires
 }
 
 QSystemNetworkInfoPrivate::~QSystemNetworkInfoPrivate()
@@ -200,14 +236,19 @@ QSystemNetworkInfoPrivate::~QSystemNetworkInfoPrivate()
         wlanSignalStrengthTimer->stop();
 }
 
-
 QSystemNetworkInfo::NetworkStatus QSystemNetworkInfoPrivate::networkStatus(QSystemNetworkInfo::NetworkMode mode)
 {
     switch(mode) {
     case QSystemNetworkInfo::GsmMode:
     case QSystemNetworkInfo::CdmaMode:
     case QSystemNetworkInfo::WcdmaMode:
-        {    
+        {
+            // radioAccessTechnology: 1 = GSM, 2 = WCDMA
+            if((radioAccessTechnology == 1 && mode != QSystemNetworkInfo::GsmMode) ||
+               (radioAccessTechnology == 2 && mode != QSystemNetworkInfo::WcdmaMode)) {
+                return QSystemNetworkInfo::NoNetworkAvailable;
+            }
+
             switch(currentCellNetworkStatus) {
                 case 0: return QSystemNetworkInfo::HomeNetwork; // CS is registered to home network
                 case 1: return QSystemNetworkInfo::Roaming; // CS is registered to some other network than home network
@@ -246,12 +287,18 @@ QSystemNetworkInfo::NetworkStatus QSystemNetworkInfoPrivate::networkStatus(QSyst
 }
 
 qint32 QSystemNetworkInfoPrivate::networkSignalStrength(QSystemNetworkInfo::NetworkMode mode)
-{ 
+{
     switch(mode) {
     case QSystemNetworkInfo::GsmMode:
     case QSystemNetworkInfo::CdmaMode:
     case QSystemNetworkInfo::WcdmaMode:
     {
+            // radioAccessTechnology: 1 = GSM, 2 = WCDMA
+            if((radioAccessTechnology == 1 && mode != QSystemNetworkInfo::GsmMode) ||
+               (radioAccessTechnology == 2 && mode != QSystemNetworkInfo::WcdmaMode)) {
+                return -1;
+            }
+
             return cellSignalStrength;
     }
     case QSystemNetworkInfo::EthernetMode: {
@@ -378,9 +425,16 @@ QString QSystemNetworkInfoPrivate::networkName(QSystemNetworkInfo::NetworkMode m
 
     case QSystemNetworkInfo::CdmaMode:
     case QSystemNetworkInfo::GsmMode:
-    case QSystemNetworkInfo::WcdmaMode:
+    case QSystemNetworkInfo::WcdmaMode: {
+        // radioAccessTechnology: 1 = GSM, 2 = WCDMA
+        if((radioAccessTechnology == 1 && mode != QSystemNetworkInfo::GsmMode) ||
+           (radioAccessTechnology == 2 && mode != QSystemNetworkInfo::WcdmaMode)) {
+            break;
+        }
+
         return currentOperatorName;
         break;
+    }
     case QSystemNetworkInfo::WimaxMode:
         break;
     default:
@@ -447,8 +501,9 @@ QNetworkInterface QSystemNetworkInfoPrivate::interfaceForMode(QSystemNetworkInfo
 
 void QSystemNetworkInfoPrivate::setupNetworkInfo()
 {
-    currentCellNetworkStatus = QSystemNetworkInfo::UndefinedStatus;
+    currentCellNetworkStatus = -1;
     currentBluetoothNetworkStatus = networkStatus(QSystemNetworkInfo::BluetoothMode);
+    currentWlanNetworkStatus = networkStatus(QSystemNetworkInfo::WlanMode);
     currentEthernetState = "down";
     currentEthernetSignalStrength = networkSignalStrength(QSystemNetworkInfo::EthernetMode);
     currentWlanSignalStrength = networkSignalStrength(QSystemNetworkInfo::WlanMode);
@@ -474,6 +529,9 @@ void QSystemNetworkInfoPrivate::setupNetworkInfo()
             rx.close();
         }
     }
+
+    currentNetworkMode = currentMode();
+
 #if !defined(QT_NO_DBUS)
     QDBusConnection systemDbusConnection = QDBusConnection::systemBus();
 
@@ -522,19 +580,7 @@ void QSystemNetworkInfoPrivate::setupNetworkInfo()
         QVariant registrationStatus = ifc4.property("RegistrationStatus");
         QString status = registrationStatus.isValid() ? registrationStatus.value<QString>() : "";
 
-        if (status ==  "Home") {
-            currentCellNetworkStatus = QSystemNetworkInfo::HomeNetwork;
-        } else if (status == "Roaming") {
-            currentCellNetworkStatus = QSystemNetworkInfo::Roaming;
-        } else if (status == "Searching") {
-            currentCellNetworkStatus = QSystemNetworkInfo::Searching;
-        } else if (status == "Offline" || status == "NoSim" || status == "PowerOff" || status == "PowerSave" || status == "NoCoverage") {
-            currentCellNetworkStatus = QSystemNetworkInfo::NoNetworkAvailable;
-        } else if (status == "Rejected") {
-            currentCellNetworkStatus = QSystemNetworkInfo::Denied;
-        } else {
-            currentCellNetworkStatus = QSystemNetworkInfo::UndefinedStatus;
-        }
+        currentCellNetworkStatus = csStatusMaemo6.value(status, -1);
 
         /* Signal handlers */
         if (!systemDbusConnection.connect(service, servicePath, "com.nokia.csd.CSNet.SignalStrength", "SignalStrengthChanged",
@@ -631,16 +677,7 @@ void QSystemNetworkInfoPrivate::setupNetworkInfo()
                        "radio_access_technology_change",
                        this, SLOT(networkModeChanged(int)))) {
         qWarning() << "unable to connect to radio_access_technology_change";
-    }   
-    if(!systemDbusConnection.connect("com.nokia.icd",
-                              "/com/nokia/icd",
-                              "com.nokia.icd",
-                              QLatin1String("status_changed"),
-                              this, SLOT(icdStatusChanged(QString,QString,QString,QString))) ) {
-        qWarning() << "unable to connect to icdStatusChanged";
     }
-    #endif /* Maemo 5 */
-
     if(!systemDbusConnection.connect("com.nokia.bme",
                               "/com/nokia/bme/signal",
                               "com.nokia.bme.signal",
@@ -655,18 +692,27 @@ void QSystemNetworkInfoPrivate::setupNetworkInfo()
                               this, SLOT(usbCableAction())) ) {
         qWarning() << "unable to connect to usbCableAction (disconnect)";
     }
+    #endif /* Maemo 5 */
+
+    if(!systemDbusConnection.connect("com.nokia.icd",
+                              "/com/nokia/icd",
+                              "com.nokia.icd",
+                              QLatin1String("status_changed"),
+                              this, SLOT(icdStatusChanged(QString,QString,QString,QString))) ) {
+        qWarning() << "unable to connect to icdStatusChanged";
+    }
     if(!systemDbusConnection.connect("org.freedesktop.Hal",
                               "/org/freedesktop/Hal/Manager",
                               "org.freedesktop.Hal.Manager",
                               QLatin1String("DeviceAdded"),
-                              this, SLOT(bluetoothNetworkStatusCheck())) ) {
+                              this, SLOT(bluetoothNetworkStatusCheck(QString))) ) {
         qWarning() << "unable to connect to bluetoothNetworkStatusCheck (1)";
     }
     if(!systemDbusConnection.connect("org.freedesktop.Hal",
                               "/org/freedesktop/Hal/Manager",
                               "org.freedesktop.Hal.Manager",
                               QLatin1String("DeviceRemoved"),
-                              this, SLOT(bluetoothNetworkStatusCheck())) ) {
+                              this, SLOT(bluetoothNetworkStatusCheck(QString))) ) {
         qWarning() << "unable to connect to bluetoothNetworkStatusCheck (2)";
     }
 #endif
@@ -712,21 +758,7 @@ void QSystemNetworkInfoPrivate::slotOperatorNameChanged(const QString &name)
 
 void QSystemNetworkInfoPrivate::slotRegistrationChanged(const QString &status)
 {
-    int newCellNetworkStatus = -1;
-
-    if (status ==  "Home") {
-        newCellNetworkStatus = QSystemNetworkInfo::HomeNetwork;
-    } else if (status == "Roaming") {
-        newCellNetworkStatus = QSystemNetworkInfo::Roaming;
-    } else if (status == "Searching") {
-        newCellNetworkStatus = QSystemNetworkInfo::Searching;
-    } else if (status == "Offline" || status == "NoSim" || status == "PowerOff" || status == "PowerSave" || status == "NoCoverage") {
-        newCellNetworkStatus = QSystemNetworkInfo::NoNetworkAvailable;
-    } else if (status == "Rejected") {
-        newCellNetworkStatus = QSystemNetworkInfo::Denied;
-    } else {
-        newCellNetworkStatus = QSystemNetworkInfo::UndefinedStatus;
-    }
+    int newCellNetworkStatus = csStatusMaemo6.value(status, -1);
 
     if (currentCellNetworkStatus != newCellNetworkStatus) {
         currentCellNetworkStatus = newCellNetworkStatus;
@@ -741,19 +773,16 @@ void QSystemNetworkInfoPrivate::slotRegistrationChanged(const QString &status)
 
 void QSystemNetworkInfoPrivate::slotCellChanged(const QString &type, int id, int lac)
 {
-    QSystemNetworkInfo::NetworkMode mode = QSystemNetworkInfo::UnknownMode;
     int newRadioAccessTechnology = 0;
     if (type == "GSM") {
-        mode = QSystemNetworkInfo::GsmMode;
         newRadioAccessTechnology = 1;
     } else if (type == "WCDMA") {
-        mode = QSystemNetworkInfo::WcdmaMode;
         newRadioAccessTechnology = 2;
     }
 
     if (newRadioAccessTechnology != radioAccessTechnology) {
         radioAccessTechnology = newRadioAccessTechnology;
-        emit networkModeChanged(mode);
+        networkModeChangeCheck();
     }
     if (currentCellId != id) {
         currentCellId = id;
@@ -784,16 +813,8 @@ void QSystemNetworkInfoPrivate::cellNetworkSignalStrengthChanged(uchar var1, uch
 
 void QSystemNetworkInfoPrivate::networkModeChanged(int newRadioAccessTechnology)
 {
-    QSystemNetworkInfo::NetworkMode newMode = QSystemNetworkInfo::UnknownMode;
     radioAccessTechnology = newRadioAccessTechnology;
-
-    if (radioAccessTechnology == 1)
-        newMode = QSystemNetworkInfo::GsmMode;
-    if (radioAccessTechnology == 2)
-        newMode = QSystemNetworkInfo::WcdmaMode;
-
-    if (newMode != QSystemNetworkInfo::UnknownMode)
-        emit networkModeChanged(newMode);
+    networkModeChangeCheck();
 }
 
 void QSystemNetworkInfoPrivate::operatorNameChanged(uchar, QString name, QString, uint, uint)
@@ -840,16 +861,22 @@ void QSystemNetworkInfoPrivate::registrationStatusChanged(uchar var1, ushort var
     }
 }
 
+#endif /* Maemo 5 */
+
 void QSystemNetworkInfoPrivate::icdStatusChanged(QString, QString var2, QString, QString)
 {
     if (var2 == "WLAN_INFRA") {
-        emit networkStatusChanged(QSystemNetworkInfo::WlanMode,
-                                  networkStatus(QSystemNetworkInfo::WlanMode));
+        QSystemNetworkInfo::NetworkStatus newWlanNetworkStatus =
+                networkStatus(QSystemNetworkInfo::WlanMode);
+
+        if (currentWlanNetworkStatus != newWlanNetworkStatus) {
+            currentWlanNetworkStatus = newWlanNetworkStatus;
+            emit networkStatusChanged(QSystemNetworkInfo::WlanMode, currentWlanNetworkStatus);
+            networkModeChangeCheck();
+        }
     }
 }
 
-#endif /* Maemo 5 */
-
 void QSystemNetworkInfoPrivate::usbCableAction()
 {
     if (currentEthernetSignalStrength != networkSignalStrength(QSystemNetworkInfo::EthernetMode)) {
@@ -866,17 +893,27 @@ void QSystemNetworkInfoPrivate::usbCableAction()
             QTextStream stream(&rx);
             stream >> newEthernetState;
             rx.close();
-            if (currentEthernetState != newEthernetState) {
-                currentEthernetState = newEthernetState;
-                emit networkStatusChanged(QSystemNetworkInfo::EthernetMode,
-                                          networkStatus(QSystemNetworkInfo::EthernetMode));
-            }
         }
     }
+
+    if (currentEthernetState != newEthernetState) {
+        currentEthernetState = newEthernetState;
+        emit networkStatusChanged(QSystemNetworkInfo::EthernetMode,
+                                  networkStatus(QSystemNetworkInfo::EthernetMode));
+    }
+
+    networkModeChangeCheck();
 }
 
 QSystemNetworkInfo::NetworkMode QSystemNetworkInfoPrivate::currentMode()
 {
+    if(networkStatus(QSystemNetworkInfo::EthernetMode) == QSystemNetworkInfo::Connected) {
+        return QSystemNetworkInfo::EthernetMode;
+    }
+    if(networkStatus(QSystemNetworkInfo::WlanMode) == QSystemNetworkInfo::Connected) {
+        return QSystemNetworkInfo::WlanMode;
+    }
+
     if (radioAccessTechnology == 1)
         return QSystemNetworkInfo::GsmMode;
     if (radioAccessTechnology == 2)
@@ -885,19 +922,35 @@ QSystemNetworkInfo::NetworkMode QSystemNetworkInfoPrivate::currentMode()
     return QSystemNetworkInfo::UnknownMode;
 }
 
+void QSystemNetworkInfoPrivate::networkModeChangeCheck()
+{
+    QSystemNetworkInfo::NetworkMode newNetworkMode = currentMode();
+    if (currentNetworkMode != newNetworkMode)
+    {
+        currentNetworkMode = newNetworkMode;
+        emit networkModeChanged(currentNetworkMode);
+    }
+}
+
 void QSystemNetworkInfoPrivate::wlanSignalStrengthCheck()
 {
-    if (currentWlanSignalStrength != networkSignalStrength(QSystemNetworkInfo::WlanMode)) {
-        currentWlanSignalStrength = networkSignalStrength(QSystemNetworkInfo::WlanMode);
+    int strength = 0;
+    strength =  networkSignalStrength(QSystemNetworkInfo::WlanMode);
+    if (currentWlanSignalStrength != strength) {
+        currentWlanSignalStrength = strength;
         emit networkSignalStrengthChanged(QSystemNetworkInfo::WlanMode, currentWlanSignalStrength);
     }
 }
 
-void QSystemNetworkInfoPrivate::bluetoothNetworkStatusCheck()
+void QSystemNetworkInfoPrivate::bluetoothNetworkStatusCheck(QString device)
 {
-    if (currentBluetoothNetworkStatus != networkStatus(QSystemNetworkInfo::BluetoothMode)) {
-        currentBluetoothNetworkStatus = networkStatus(QSystemNetworkInfo::BluetoothMode);
-        emit networkStatusChanged(QSystemNetworkInfo::BluetoothMode, currentBluetoothNetworkStatus);
+    if (device == "/org/freedesktop/Hal/devices/net_1b") {
+        QTimer::singleShot(500, this, SLOT(usbCableAction()));
+    } else {
+        if (currentBluetoothNetworkStatus != networkStatus(QSystemNetworkInfo::BluetoothMode)) {
+            currentBluetoothNetworkStatus = networkStatus(QSystemNetworkInfo::BluetoothMode);
+            emit networkStatusChanged(QSystemNetworkInfo::BluetoothMode, currentBluetoothNetworkStatus);
+        }
     }
 }
 
@@ -928,7 +981,10 @@ QSystemDisplayInfoPrivate::~QSystemDisplayInfoPrivate()
 
 int QSystemDisplayInfoPrivate::displayBrightness(int screen)
 {
-    Q_UNUSED(screen);
+    QDesktopWidget wid;
+    if(wid.screenCount() - 1 < screen) {
+        return -1;
+    }
     GConfItem currentBrightness("/system/osso/dsm/display/display_brightness");
     GConfItem maxBrightness("/system/osso/dsm/display/max_display_brightness_levels");
     if(maxBrightness.value().toInt()) {
@@ -952,11 +1008,9 @@ QSystemStorageInfoPrivate::~QSystemStorageInfoPrivate()
 QSystemDeviceInfoPrivate::QSystemDeviceInfoPrivate(QSystemDeviceInfoLinuxCommonPrivate *parent)
         : QSystemDeviceInfoLinuxCommonPrivate(parent)
 {
-    setConnection();
     flightMode = false;
  #if !defined(QT_NO_DBUS)
     previousPowerState = QSystemDeviceInfo::UnknownPower;
-    setupBluetooth();
     setupProfile();
 #endif
 }
@@ -969,21 +1023,24 @@ QSystemDeviceInfoPrivate::~QSystemDeviceInfoPrivate()
 void QSystemDeviceInfoPrivate::halChanged(int,QVariantList map)
 {
     for(int i=0; i < map.count(); i++) {
-//       qWarning() << __FUNCTION__ << map.at(i).toString();
+       qWarning() << __FUNCTION__ << map.at(i).toString();
        if(map.at(i).toString() == "battery.charge_level.percentage") {
             int level = batteryLevel();
             emit batteryLevelChanged(level);
+            QSystemDeviceInfo::BatteryStatus stat = QSystemDeviceInfo::NoBatteryLevel;
+
             if(level < 4) {
-                emit batteryStatusChanged(QSystemDeviceInfo::BatteryCritical);
+                stat = QSystemDeviceInfo::BatteryCritical;
             } else if(level < 11) {
-                emit batteryStatusChanged(QSystemDeviceInfo::BatteryVeryLow);
+                stat = QSystemDeviceInfo::BatteryVeryLow;
             } else if(level < 41) {
-                emit batteryStatusChanged(QSystemDeviceInfo::BatteryLow);
+                stat = QSystemDeviceInfo::BatteryLow;
             } else if(level > 40) {
-                emit batteryStatusChanged(QSystemDeviceInfo::BatteryNormal);
+                stat = QSystemDeviceInfo::BatteryNormal;
             }
-            else {
-                emit batteryStatusChanged(QSystemDeviceInfo::NoBatteryLevel);
+            if(currentBatStatus != stat) {
+                currentBatStatus = stat;
+                Q_EMIT batteryStatusChanged(stat);
             }
         }
         if((map.at(i).toString() == "maemo.charger.connection_status")
@@ -992,7 +1049,8 @@ void QSystemDeviceInfoPrivate::halChanged(int,QVariantList map)
             if (previousPowerState != state)
                 emit powerStateChanged(state);
             previousPowerState = state;
-       }} //end map
+       }
+    } //end map
 }
 #endif
 
@@ -1002,9 +1060,12 @@ QSystemDeviceInfo::Profile QSystemDeviceInfoPrivate::currentProfile()
     if (flightMode)
         return QSystemDeviceInfo::OfflineProfile;
 
-    if (silentProfile )
+    if (silentProfile)
         return vibratingAlertEnabled ? QSystemDeviceInfo::VibProfile : QSystemDeviceInfo::SilentProfile;
 
+    if (beepProfile)
+        return QSystemDeviceInfo::BeepProfile;
+
     if (ringingAlertVolume > 75)
         return QSystemDeviceInfo::LoudProfile;
 
@@ -1147,6 +1208,7 @@ QSystemDeviceInfo::PowerState QSystemDeviceInfoPrivate::currentPowerState()
 
 void QSystemDeviceInfoPrivate::setupProfile()
 {
+    qDebug() << Q_FUNC_INFO;
     QDBusConnection systemDbusConnection = QDBusConnection::systemBus();
 
     QDBusInterface mceConnectionInterface("com.nokia.mce",
@@ -1158,7 +1220,7 @@ void QSystemDeviceInfoPrivate::setupProfile()
         return;
     } else {
         QDBusReply<QString> deviceModeReply = mceConnectionInterface.call("get_device_mode");
-        flightMode = deviceModeReply.value() == "flight";
+        flightMode = QString::compare(deviceModeReply.value(), "flight", Qt::CaseInsensitive) == 0;
     }
 
     if (!systemDbusConnection.connect("com.nokia.mce",
@@ -1182,14 +1244,19 @@ void QSystemDeviceInfoPrivate::setupProfile()
     QDBusReply<QString> profileNameReply = connectionInterface.call("get_profile");
     if (profileNameReply.isValid())
         profileName = profileNameReply.value();
+qDebug() << Q_FUNC_INFO << profileName;
 
     QDBusReply<QString> ringingAlertTypeReply = connectionInterface.call("get_value", profileName, "ringing.alert.type");
-    if (ringingAlertTypeReply.isValid())
-        silentProfile = ringingAlertTypeReply.value() == "silent";
+    qDebug() << ringingAlertTypeReply.value();
+
+    if (ringingAlertTypeReply.isValid()) {
+        silentProfile = QString::compare(ringingAlertTypeReply.value(), "silent", Qt::CaseInsensitive) == 0;
+        beepProfile = QString::compare(ringingAlertTypeReply.value(), "beep", Qt::CaseInsensitive) == 0;
+    }
 
     QDBusReply<QString> vibratingAlertEnabledReply = connectionInterface.call("get_value", profileName, "vibrating.alert.enabled");
     if (vibratingAlertEnabledReply.isValid())
-        vibratingAlertEnabled = vibratingAlertEnabledReply.value() == "On";
+        vibratingAlertEnabled = QString::compare(vibratingAlertEnabledReply.value(), "On", Qt::CaseInsensitive) == 0;
 
     QDBusReply<QString> ringingAlertVolumeReply = connectionInterface.call("get_value", profileName, "ringing.alert.volume");
     if (ringingAlertVolumeReply.isValid())
@@ -1219,82 +1286,142 @@ void QSystemDeviceInfoPrivate::deviceModeChanged(QString newMode)
 
 void QSystemDeviceInfoPrivate::profileChanged(bool changed, bool active, QString profile, QList<ProfileDataValue> values)
 {
+    qDebug() << __FUNCTION__;
     if (active) {
-        const QSystemDeviceInfo::Profile previousProfile = currentProfile();
         profileName = profile;
         foreach (const ProfileDataValue value, values) {
-            if (value.key == "ringing.alert.type")
-                silentProfile = value.val == "silent";
+            qDebug() << value.key << value.val;
+            if (value.key == "ringing.alert.type") {
+                silentProfile = QString::compare(value.val, "silent", Qt::CaseInsensitive) == 0;
+                beepProfile = QString::compare(value.val, "beep", Qt::CaseInsensitive) == 0;
+            }
             else if (value.key == "vibrating.alert.enabled")
-                vibratingAlertEnabled = value.val == "On";
+                vibratingAlertEnabled = QString::compare(value.val, "On", Qt::CaseInsensitive) == 0;
             else if (value.key == "ringing.alert.volume")
                 ringingAlertVolume = value.val.toInt();
         }
-        QSystemDeviceInfo::Profile newProfile = currentProfile();
-        if (previousProfile != newProfile)
-           emit currentProfileChanged(newProfile);
+        if (changed)
+            emit currentProfileChanged(currentProfile());
     }
 }
 
+QString QSystemDeviceInfoPrivate::model()
+{
+    QString name;
+    if(productName()== "RX-51")
+        return "N900";
+
+    name = "Harmattan"; //fake this for now
+
+    return name;
+
+}
+
+QString QSystemDeviceInfoPrivate::productName()
+{
+#if !defined(QT_NO_DBUS)
+#if defined(Q_WS_MAEMO_6)
+    QString dBusService = "com.nokia.SystemInfo";
+#else
+    /* Maemo 5 */
+    QString dBusService = "com.nokia.SystemInfo";
+#endif
+    QDBusInterface connectionInterface(dBusService,
+                                       "/com/nokia/SystemInfo",
+                                       "com.nokia.SystemInfo",
+                                       QDBusConnection::systemBus());
+
+    QDBusReply< QByteArray > reply = connectionInterface.call("GetConfigValue","/component/product");
+    return reply.value();
+#endif
+}
+
 #endif
 
 //////////////
 ///////
 QSystemScreenSaverPrivate::QSystemScreenSaverPrivate(QObject *parent)
-        : QSystemScreenSaverLinuxCommonPrivate(parent), m_screenSaverInhibited(false)
-
+        : QSystemScreenSaverLinuxCommonPrivate(parent),isInhibited(0)
 {
     ssTimer = new QTimer(this);
+#if !defined(QT_NO_DBUS)
+    mceConnectionInterface = new QDBusInterface("com.nokia.mce",
+                                                "/com/nokia/mce/request",
+                                                "com.nokia.mce.request",
+                                                QDBusConnection::systemBus());
+#endif
 }
 
 QSystemScreenSaverPrivate::~QSystemScreenSaverPrivate()
 {
-     if(ssTimer->isActive())
-         ssTimer->stop();
-
-     m_screenSaverInhibited = false;
-}
-
-bool QSystemScreenSaverPrivate::screenSaverInhibited()
-{
-    return m_screenSaverInhibited;
+    if (ssTimer->isActive()) {
+        ssTimer->stop();
+    }
+#if !defined(QT_NO_DBUS)
+    delete mceConnectionInterface, mceConnectionInterface = 0;
+#endif
 }
 
 bool QSystemScreenSaverPrivate::setScreenSaverInhibit()
 {
-    if (m_screenSaverInhibited)
-        return true;
-
-     m_screenSaverInhibited = true;
-     display_blanking_pause();
-     connect(ssTimer, SIGNAL(timeout()), this, SLOT(display_blanking_pause()));
-     ssTimer->start(3000); //3 seconds interval
-     return true;
+    wakeUpDisplay();
+    if (!ssTimer->isActive()) {
+        connect(ssTimer, SIGNAL(timeout()), this, SLOT(wakeUpDisplay()));
+        // Set a wake up interval of 30 seconds.
+        // The reason for this is to avoid the situation where
+        // a crashed/hung application keeps the display on.
+        ssTimer->start(30000);
+        isInhibited = true;
+    } else {
+        isInhibited = false;
+    }
+     return screenSaverInhibited();
 }
 
-void QSystemScreenSaverPrivate::display_blanking_pause()
+void QSystemScreenSaverPrivate::wakeUpDisplay()
 {
 #if !defined(QT_NO_DBUS)
-    QDBusInterface connectionInterface("com.nokia.mce",
-                                       "/com/nokia/mce/request",
-                                       "com.nokia.mce.request",
-                                       QDBusConnection::systemBus());
-    if (!connectionInterface.isValid()) {
-        qWarning() << "interface not valid";
-        return;
+    if (mceConnectionInterface->isValid()) {
+        mceConnectionInterface->call("req_tklock_mode_change", "unlocked");
+        mceConnectionInterface->call("req_display_blanking_pause");
     }
-    connectionInterface.call("req_display_blanking_pause");
 #endif
 }
 
-bool QSystemScreenSaverPrivate::isScreenLockEnabled()
+bool QSystemScreenSaverPrivate::screenSaverInhibited()
 {
-   return false;
-}
+    bool displayOn = false;
+    GConfItem screenBlankItem("/system/osso/dsm/display/inhibit_blank_mode");
+    /* 0 - no inhibit
+       1 - inhibit dim with charger
+       2 - inhibit blank with charger (display still dims)
+       3 - inhibit dim (always)
+       4 - inhibit blank (always; display still dims)
+*/
+    int blankingItem = screenBlankItem.value().toInt();
 
-bool QSystemScreenSaverPrivate::isScreenSaverActive()
-{
-    return false;
+    bool isBlankingInhibited = false;
+    QSystemDeviceInfo devInfo(this);
+    QSystemDeviceInfo::PowerState batState = devInfo.currentPowerState();
+
+    if( ((batState == QSystemDeviceInfo::WallPower || batState == QSystemDeviceInfo::WallPowerChargingBattery)
+       && blankingItem == 2) || blankingItem == 4) {
+        isBlankingInhibited = true;
+    }
+
+#if !defined(QT_NO_DBUS)
+    if (mceConnectionInterface->isValid()) {
+        // The most educated guess for the screen saver being inhibited is to determine
+        // whether the display is on. That is because the QSystemScreenSaver cannot
+        // prevent other processes from blanking the screen (like, if
+        // MCE decides to blank the screen for some reason).
+        // but that means it reports to be inhibited when display is on. meaning
+        // effectly always inhibited by default. so we try a bit harder
+        QDBusReply<QString> reply = mceConnectionInterface->call("get_display_status");
+        displayOn = ("on" == reply.value());
+    }
+#endif
+    return (displayOn && isBlankingInhibited && isInhibited);
 }
 
 #include "moc_qsysteminfo_maemo_p.cpp"
