@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,18 +33,12 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
-#include "qsysteminfo.h"
+#include "qsysteminfocommon_p.h"
 #include "qsysteminfo_s60_p.h"
 
 #include <QStringList>
@@ -100,7 +94,7 @@ QStringList QSystemInfoPrivate::availableLanguages() const
 {
     QStringList languages;
     QList<HbInputLanguage> hblanguages = HbKeymapFactory::availableLanguages();
-    foreach(HbInputLanguage lang, hblanguages) {       
+    foreach(HbInputLanguage lang, hblanguages) {
         QString language = QLocaleToISO639_1(lang.language());
         if (!language.isEmpty()) {
             languages << language;
@@ -108,7 +102,7 @@ QStringList QSystemInfoPrivate::availableLanguages() const
     }
     languages.removeDuplicates();
     return languages;
-}  
+}
 #else
 QStringList QSystemInfoPrivate::availableLanguages() const
 {
@@ -132,7 +126,7 @@ QStringList QSystemInfoPrivate::availableLanguages() const
 }
 #endif //HB_SUPPORTED
 
-QString QSystemInfoPrivate::QLocaleToISO639_1(QLocale::Language language) const    
+QString QSystemInfoPrivate::QLocaleToISO639_1(QLocale::Language language) const
 {
        switch(language) {
        case QLocale::English: return "en";
@@ -178,11 +172,11 @@ QString QSystemInfoPrivate::QLocaleToISO639_1(QLocale::Language language) const
        case QLocale::Galician: return "gl";
        case QLocale::Basque: return "eu";
        case QLocale::Marathi: return "mr";
-       case QLocale::Korean: return "ko";       
+       case QLocale::Korean: return "ko";
        default:
            break;
        }
-    return "";        
+    return "";
 }
 
 QString QSystemInfoPrivate::TLanguageToISO639_1(TLanguage language) const
@@ -714,15 +708,85 @@ int QSystemDisplayInfoPrivate::colorDepth(int screen)
     return depth;
 }
 
+
+// QSystemDisplayInfo::DisplayOrientation QSystemDisplayInfoPrivate::getOrientation(int screen)
+// {
+//     QSystemDisplayInfo::DisplayOrientation orientation = QSystemDisplayInfo::Unknown;
+
+//     if(screen < 16 && screen > -1) {
+//         int rotation = 0;
+//         switch(rotation) {
+//         case 0:
+//         case 360:
+//             orientation = QSystemDisplayInfo::Landscape;
+//             break;
+//         case 90:
+//             orientation = QSystemDisplayInfo::Portrait;
+//             break;
+//         case 180:
+//             orientation = QSystemDisplayInfo::InvertedLandscape;
+//             break;
+//         case 270:
+//             orientation = QSystemDisplayInfo::InvertedPortrait;
+//             break;
+//         };
+//     }
+//     return orientation;
+// }
+
+
+// float QSystemDisplayInfoPrivate::contrast(int screen)
+// {
+//     Q_UNUSED(screen);
+
+//     return 0.0;
+// }
+
+// int QSystemDisplayInfoPrivate::getDPIWidth(int screen)
+// {
+//     int dpi=0;
+//     if(screen < 16 && screen > -1) {
+
+//         }
+//     return dpi;
+// }
+
+// int QSystemDisplayInfoPrivate::getDPIHeight(int screen)
+// {
+//     int dpi=0;
+//     if(screen < 16 && screen > -1) {
+
+//     }
+//     return dpi;
+// }
+
+
+// int QSystemDisplayInfoPrivate::physicalHeight(int screen)
+// {
+//     int height=0;
+
+//     return height;
+// }
+
+// int QSystemDisplayInfoPrivate::physicalWidth(int screen)
+// {
+//     int width=0;
+
+//     return width;
+// }
+
+
 QSystemStorageInfoPrivate::QSystemStorageInfoPrivate(QObject *parent)
     : QObject(parent)
 {
     iFs.Connect();
+    DeviceInfo::instance()->mmcStorageStatus()->addObserver(this);
 }
 
 QSystemStorageInfoPrivate::~QSystemStorageInfoPrivate()
 {
     iFs.Close();
+    DeviceInfo::instance()->mmcStorageStatus()->removeObserver(this);
 }
 
 qlonglong QSystemStorageInfoPrivate::totalDiskSpace(const QString &driveVolume)
@@ -816,6 +880,11 @@ QSystemStorageInfo::DriveType QSystemStorageInfoPrivate::typeForDrive(const QStr
     }
 
     return QSystemStorageInfo::NoDrive;
+}
+
+void QSystemStorageInfoPrivate::storageStatusChanged(bool added, const QString &aDriveVolume)
+{
+    emit logicalDriveChanged(added, aDriveVolume);
 };
 
 QSystemDeviceInfoPrivate::QSystemDeviceInfoPrivate(QObject *parent)
@@ -1031,6 +1100,10 @@ QSystemDeviceInfo::BatteryStatus QSystemDeviceInfoPrivate::batteryStatus()
 
 QSystemDeviceInfo::SimStatus QSystemDeviceInfoPrivate::simStatus()
 {
+#ifdef SYMBIAN_3_1
+    if (!DeviceInfo::instance()->subscriberInfo()->imsi().isEmpty())
+        return QSystemDeviceInfo::SingleSimAvailable;
+#else //SYMBIAN_3_1
     TInt lockStatus = 0;
     TInt err = RProperty::Get(KPSUidStartup, KStartupSimLockStatus, lockStatus);
     if (err == KErrNone && (TPSSimLockStatus)lockStatus != ESimLockOk) {
@@ -1042,7 +1115,7 @@ QSystemDeviceInfo::SimStatus QSystemDeviceInfoPrivate::simStatus()
     if (err == KErrNone && TPSSimStatus(simStatus) == ESimUsable) {
         return QSystemDeviceInfo::SingleSimAvailable;
     }
-
+#endif //SYMBIAN_3_1
     return QSystemDeviceInfo::SimNotAvailable;
 }
 
@@ -1085,6 +1158,18 @@ void QSystemDeviceInfoPrivate::chargingStatusChanged()
     emit powerStateChanged(currentPowerState());
 }
 
+bool QSystemDeviceInfoPrivate::currentBluetoothPowerState()
+{
+    TInt btPowerState = EBTPowerOff;
+    TRAP_IGNORE(
+    CRepository* crep = CRepository::NewLC(KCRUidBluetoothPowerState);
+    User::LeaveIfError(crep->Get(KBTPowerState, btPowerState));
+    CleanupStack::PopAndDestroy(crep);
+    )
+
+    return btPowerState == EBTPowerOn;
+}
+
 DeviceInfo *DeviceInfo::m_instance = NULL;
 
 QSystemScreenSaverPrivate::QSystemScreenSaverPrivate(QObject *parent)
