@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,23 +33,18 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
-#include "qversitdefs_p.h"
+#include "qversitcontactsdefs_p.h"
 #include "qversitcontactimporter_p.h"
 #include "qversitdocument.h"
 #include "qversitproperty.h"
 #include "qmobilityglobal.h"
+#include "qmobilitypluginsearch.h"
 
 #include <qcontactmanagerengine.h>
 #include <qcontact.h>
@@ -74,6 +69,8 @@
 #include <qcontactdisplaylabel.h>
 #include <qcontactthumbnail.h>
 #include <qcontactringtone.h>
+#include "qversitcontacthandler.h"
+#include "qversitcontactpluginloader_p.h"
 
 #include <QHash>
 #include <QFile>
@@ -83,23 +80,24 @@ QTM_USE_NAMESPACE
 /*!
  * Constructor.
  */
-QVersitContactImporterPrivate::QVersitContactImporterPrivate() :
+QVersitContactImporterPrivate::QVersitContactImporterPrivate(const QStringList& profiles) :
     mPropertyHandler(NULL),
-    mDefaultResourceHandler(new QVersitDefaultResourceHandler)
+    mPropertyHandler2(NULL),
+    mPropertyHandlerVersion(0),
+    mDefaultResourceHandler(new QVersitDefaultResourceHandler),
+    mResourceHandler(mDefaultResourceHandler)
 {
-    mResourceHandler = mDefaultResourceHandler;
-
     // Contact detail mappings
     int versitPropertyCount =
-        sizeof(versitContactDetailMappings)/sizeof(VersitContactDetailMapping);
+        sizeof(versitContactDetailMappings)/sizeof(VersitDetailMapping);
     for (int i=0; i < versitPropertyCount; i++) {
         QString versitPropertyName =
             QLatin1String(versitContactDetailMappings[i].versitPropertyName);
         QPair<QString,QString> contactDetail;
         contactDetail.first =
-            QLatin1String(versitContactDetailMappings[i].contactDetailDefinitionName);
+            QLatin1String(versitContactDetailMappings[i].detailDefinitionName);
         contactDetail.second =
-            QLatin1String(versitContactDetailMappings[i].contactDetailValueKey);
+            QLatin1String(versitContactDetailMappings[i].detailFieldName);
         mDetailMappings.insert(versitPropertyName,contactDetail);
     }
 
@@ -118,6 +116,8 @@ QVersitContactImporterPrivate::QVersitContactImporterPrivate() :
             QLatin1String(versitSubTypeMappings[i].versitString),
             QLatin1String(versitSubTypeMappings[i].contactString));
     }
+
+    mPluginPropertyHandlers = QVersitContactPluginLoader::instance()->createContactHandlers(profiles);
 }
 
 /*!
@@ -126,6 +126,9 @@ QVersitContactImporterPrivate::QVersitContactImporterPrivate() :
 QVersitContactImporterPrivate::~QVersitContactImporterPrivate()
 {
     delete mDefaultResourceHandler;
+    foreach (QVersitContactHandler* pluginHandler, mPluginPropertyHandlers) {
+        delete pluginHandler;
+    }
 }
 
 /*!
@@ -135,7 +138,8 @@ bool QVersitContactImporterPrivate::importContact(
         const QVersitDocument& document, int contactIndex, QContact* contact,
         QVersitContactImporter::Error* error)
 {
-    if (document.type() != QVersitDocument::VCard21Type
+    if (document.componentType() != QLatin1String("VCARD")
+        && document.type() != QVersitDocument::VCard21Type
         && document.type() != QVersitDocument::VCard30Type) {
         *error = QVersitContactImporter::InvalidDocumentError;
         return false;
@@ -161,64 +165,96 @@ bool QVersitContactImporterPrivate::importContact(
 
     contact->setType(QContactType::TypeContact);
     QContactManagerEngine::setContactDisplayLabel(contact, QVersitContactImporterPrivate::synthesizedDisplayLabel(*contact));
+
+    // run plugin handlers
+    foreach (QVersitContactImporterPropertyHandlerV2* handler, mPluginPropertyHandlers) {
+        handler->documentProcessed(document, contact);
+    }
+    // run the v2 handler, if set
+    if (mPropertyHandler2 && mPropertyHandlerVersion > 1) {
+        mPropertyHandler2->documentProcessed(document, contact);
+    }
+
     return true;
 }
 
 void QVersitContactImporterPrivate::importProperty(
         const QVersitDocument& document, const QVersitProperty& property, int contactIndex,
-        QContact* contact) const
+        QContact* contact)
 {
     if (mPropertyHandler
+        && mPropertyHandlerVersion == 1
         && mPropertyHandler->preProcessProperty(document, property, contactIndex, contact))
         return;
 
     QPair<QString,QString> detailDefinition =
         mDetailMappings.value(property.name());
     QString detailDefinitionName = detailDefinition.first;
+
+    QList<QContactDetail> updatedDetails;
+
     bool success = false;
+
+    // The following functions create and save the details to the contact
     if (detailDefinitionName == QContactAddress::DefinitionName) {
-        success = createAddress(property, contact);
+        success = createAddress(property, contact, &updatedDetails); // pass in group
     } else if (detailDefinitionName == QContactName::DefinitionName) {
-        success = createName(property, contact);
+        success = createName(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactBirthday::DefinitionName) {
-        success = createBirthday(property, contact);
+        success = createBirthday(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactGeoLocation::DefinitionName){
-        success = createGeoLocation(property, contact);
+        success = createGeoLocation(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactOrganization::DefinitionName) {
-        success = createOrganization(property, contact);
+        success = createOrganization(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactNickname::DefinitionName) {
-        success = createNicknames(property, contact);
+        success = createNicknames(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactRingtone::DefinitionName) {
-        success = createRingtone(property, contact);
+        success = createRingtone(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactThumbnail::DefinitionName) {
-        success = createThumbnail(property, contact);
+        success = createThumbnail(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactTimestamp::DefinitionName) {
-        success = createTimeStamp(property, contact);
+        success = createTimeStamp(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactPhoneNumber::DefinitionName) {
-        success = createPhone(property, contact);
+        success = createPhone(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactAnniversary::DefinitionName) {
-        success = createAnniversary(property, contact);
+        success = createAnniversary(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactFamily::DefinitionName) {
-        success = createFamily(property, contact);
+        success = createFamily(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactOnlineAccount::DefinitionName) {
-        success = createOnlineAccount(property, contact);
+        success = createOnlineAccount(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactTag::DefinitionName) {
-        success = createTags(property, contact);
+        success = createTags(property, contact, &updatedDetails);
     } else if (detailDefinitionName == QContactDisplayLabel::DefinitionName) {
-        success = createCustomLabel(property, contact);
+        success = createCustomLabel(property, contact, &updatedDetails);
     } else {
         // Look up mDetailMappings for a simple mapping from property to detail.
-        success = createNameValueDetail(property, contact);
+        success = createNameValueDetail(property, contact, &updatedDetails);
     }
 
-    if (mPropertyHandler)
+    // run plugin handlers
+    foreach (QVersitContactImporterPropertyHandlerV2* handler, mPluginPropertyHandlers) {
+        handler->propertyProcessed(document, property, *contact, &success, &updatedDetails);
+    }
+    // run the v2 handler, if set
+    if (mPropertyHandler2 && mPropertyHandlerVersion > 1) {
+        mPropertyHandler2->propertyProcessed(document, property, *contact, &success, &updatedDetails);
+    }
+
+    foreach (QContactDetail detail, updatedDetails) {
+        contact->saveDetail(&detail);
+    }
+
+    // run the v1 handler, if set
+    if (mPropertyHandler && mPropertyHandlerVersion == 1)
         mPropertyHandler->postProcessProperty(document, property, success, contactIndex, contact);
 }
 /*!
  * Creates a QContactName from \a property
  */
 bool QVersitContactImporterPrivate::createName(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
     QContactName name;
     QContactDetail detail = contact->detail(QContactName::DefinitionName);
@@ -252,7 +288,7 @@ bool QVersitContactImporterPrivate::createName(
     if (!value.isEmpty())
         name.setSuffix(value);
 
-    saveDetailWithContext(contact, &name, extractContexts(property));
+    saveDetailWithContext(updatedDetails, name, extractContexts(property));
     return true;
 }
 
@@ -260,8 +296,11 @@ bool QVersitContactImporterPrivate::createName(
  * Creates a QContactPhoneNumber from \a property
  */
 bool QVersitContactImporterPrivate::createPhone(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QContactPhoneNumber phone;
     QString value(property.value());
     if (value.isEmpty())
@@ -273,7 +312,7 @@ bool QVersitContactImporterPrivate::createPhone(
     if (!subTypes.isEmpty())
         phone.setSubTypes(subTypes);
 
-    saveDetailWithContext(contact, &phone, extractContexts(property));
+    saveDetailWithContext(updatedDetails, phone, extractContexts(property));
     return true;
 }
 
@@ -281,8 +320,11 @@ bool QVersitContactImporterPrivate::createPhone(
  * Creates a QContactAddress from \a property
  */
 bool QVersitContactImporterPrivate::createAddress(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QContactAddress address;
 
     QVariant variant = property.variantValue();
@@ -315,7 +357,7 @@ bool QVersitContactImporterPrivate::createAddress(
     if (!subTypes.isEmpty())
         address.setSubTypes(subTypes);
 
-    saveDetailWithContext(contact, &address, extractContexts(property));
+    saveDetailWithContext(updatedDetails, address, extractContexts(property));
     return true;
 }
 
@@ -323,7 +365,9 @@ bool QVersitContactImporterPrivate::createAddress(
  * Creates a QContactOrganization from \a property
  */
 bool QVersitContactImporterPrivate::createOrganization(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
     QContactOrganization organization;
     QPair<QString,QString> detailNameAndFieldName =
@@ -350,7 +394,7 @@ bool QVersitContactImporterPrivate::createOrganization(
         return false;
     }
 
-    saveDetailWithContext(contact, &organization, extractContexts(property));
+    saveDetailWithContext(updatedDetails, organization, extractContexts(property));
     return true;
 }
 
@@ -389,8 +433,11 @@ void QVersitContactImporterPrivate::setOrganizationLogo(
  * Creates a QContactTimeStamp from \a property
  */
 bool QVersitContactImporterPrivate::createTimeStamp(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QContactTimestamp timeStamp;
     QString value(property.value());
     bool utc = value.endsWith(QLatin1Char('Z'), Qt::CaseInsensitive);
@@ -403,7 +450,7 @@ bool QVersitContactImporterPrivate::createTimeStamp(
     if (utc)
         dateTime.setTimeSpec(Qt::UTC);
     timeStamp.setLastModified(dateTime);
-    saveDetailWithContext(contact, &timeStamp, extractContexts(property));
+    saveDetailWithContext(updatedDetails, timeStamp, extractContexts(property));
     return true;
 }
 
@@ -411,14 +458,17 @@ bool QVersitContactImporterPrivate::createTimeStamp(
  * Creates a QContactAnniversary from \a property
  */
 bool QVersitContactImporterPrivate::createAnniversary(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QContactAnniversary anniversary;
     QDateTime dateTime = parseDateTime(property.value(), QLatin1String("yyyyMMdd"));
     if (!dateTime.isValid())
         return false;
     anniversary.setOriginalDate(dateTime.date());
-    saveDetailWithContext(contact, &anniversary, extractContexts(property));
+    saveDetailWithContext(updatedDetails, anniversary, extractContexts(property));
     return true;
 }
 
@@ -426,56 +476,79 @@ bool QVersitContactImporterPrivate::createAnniversary(
  * Creates a QContactBirthday from \a property
  */
 bool QVersitContactImporterPrivate::createBirthday(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QContactBirthday bday;
     QDateTime dateTime = parseDateTime(property.value(), QLatin1String("yyyyMMdd"));
     if (!dateTime.isValid())
         return false;
     bday.setDate(dateTime.date());
-    saveDetailWithContext(contact, &bday, extractContexts(property));
+    saveDetailWithContext(updatedDetails, bday, extractContexts(property));
     return true;
 }
 
 /*!
- * Creates QContactNicknames from \a property and adds them to \a contact
+ * Creates QContactNicknames from \a property
  */
 bool QVersitContactImporterPrivate::createNicknames(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QVariant variant = property.variantValue();
     if (property.valueType() != QVersitProperty::ListType
             || variant.type() != QVariant::StringList)
         return false;
     QStringList values = variant.toStringList();
     QStringList contexts = extractContexts(property);
+
+    // We don't want to make duplicates of existing nicknames
+    QSet<QString> existingNicknames;
+    foreach (const QContactNickname& nickname, contact->details<QContactNickname>()) {
+        existingNicknames.insert(nickname.nickname());
+    }
     foreach(const QString& value, values) {
-        if (!value.isEmpty()) {
-            QContactNickname nickName;
-            nickName.setNickname(value);
-            saveDetailWithContext(contact, &nickName, contexts);
+        if (!value.isEmpty() && !existingNicknames.contains(value)) {
+            QContactNickname nickname;
+            nickname.setNickname(value);
+            saveDetailWithContext(updatedDetails, nickname, contexts);
+            existingNicknames.insert(value);
         }
     }
     return true;
 }
 
 /*!
- * Creates QContactTags from \a property and adds them to \a contact
+ * Creates QContactTags from \a property
  */
 bool QVersitContactImporterPrivate::createTags(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QVariant variant = property.variantValue();
     if (property.valueType() != QVersitProperty::ListType
             || variant.type() != QVariant::StringList)
         return false;
     QStringList values = variant.toStringList();
     QStringList contexts = extractContexts(property);
+
+    // We don't want to make duplicates of existing tags
+    QSet<QString> existingTags;
+    foreach (const QContactTag& tag, contact->details<QContactTag>()) {
+        existingTags.insert(tag.tag());
+    }
     foreach(const QString& value, values) {
-        if (!value.isEmpty()) {
+        if (!value.isEmpty() && !existingTags.contains(value)) {
             QContactTag tag;
             tag.setTag(value);
-            saveDetailWithContext(contact, &tag, contexts);
+            saveDetailWithContext(updatedDetails, tag, contexts);
+            existingTags.insert(value);
         }
     }
     return true;
@@ -485,8 +558,11 @@ bool QVersitContactImporterPrivate::createTags(
  * Creates a QContactOnlineAccount from \a property
  */
 bool QVersitContactImporterPrivate::createOnlineAccount(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QContactOnlineAccount onlineAccount;
     QString value(property.value());
     if (value.isEmpty())
@@ -503,19 +579,25 @@ bool QVersitContactImporterPrivate::createOnlineAccount(
         onlineAccount.setSubTypes(QContactOnlineAccount::SubTypeImpp);
     }
 
-    saveDetailWithContext(contact, &onlineAccount, extractContexts(property));
+    saveDetailWithContext(updatedDetails, onlineAccount, extractContexts(property));
     return true;
 }
 
-bool QVersitContactImporterPrivate::createRingtone(const QVersitProperty &property,
-                                                   QContact *contact) const
+/*!
+ * Creates a QContactRingtone from \a property
+ */
+bool QVersitContactImporterPrivate::createRingtone(
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QString location;
     QByteArray data;
     if (saveDataFromProperty(property, &location, &data) && !location.isEmpty()) {
         QContactRingtone ringtone;
         ringtone.setAudioRingtoneUrl(location);
-        saveDetailWithContext(contact, &ringtone, extractContexts(property));
+        saveDetailWithContext(updatedDetails, ringtone, extractContexts(property));
         return true;
     }
     return false;
@@ -525,7 +607,9 @@ bool QVersitContactImporterPrivate::createRingtone(const QVersitProperty &proper
  * Creates a QContactAvatar from \a property
  */
 bool QVersitContactImporterPrivate::createThumbnail(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
     QString location;
     QByteArray data;
@@ -534,7 +618,7 @@ bool QVersitContactImporterPrivate::createThumbnail(
     if (saveDataFromProperty(property, &location, &data) && !location.isEmpty()) {
         QContactAvatar avatar;
         avatar.setImageUrl(location);
-        saveDetailWithContext(contact, &avatar, extractContexts(property));
+        saveDetailWithContext(updatedDetails, avatar, extractContexts(property));
         success = true;
     }
     if (!data.isEmpty()) {
@@ -545,7 +629,7 @@ bool QVersitContactImporterPrivate::createThumbnail(
             if (thumbnail.isEmpty() || image.byteCount() < thumbnail.thumbnail().byteCount()) {
                 thumbnail.setThumbnail(image);
             }
-            saveDetailWithContext(contact, &thumbnail, extractContexts(property));
+            saveDetailWithContext(updatedDetails, thumbnail, extractContexts(property));
             success = true;
         }
     }
@@ -557,8 +641,11 @@ bool QVersitContactImporterPrivate::createThumbnail(
  * Creates a QContactGeoLocation from \a property
  */
 bool QVersitContactImporterPrivate::createGeoLocation(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QContactGeoLocation geo;
     QVariant variant = property.variantValue();
     if (property.valueType() != QVersitProperty::CompoundType
@@ -571,7 +658,7 @@ bool QVersitContactImporterPrivate::createGeoLocation(
     geo.setLatitude(takeFirst(values).toDouble(&ok2));
 
     if (ok1 && ok2) {
-        saveDetailWithContext(contact, &geo, extractContexts(property));
+        saveDetailWithContext(updatedDetails, geo, extractContexts(property));
         return true;
     } else {
         return false;
@@ -582,7 +669,9 @@ bool QVersitContactImporterPrivate::createGeoLocation(
  * Creates a QContactFamily from \a property
  */
 bool QVersitContactImporterPrivate::createFamily(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
     QString val = property.value();
     QContactFamily family = contact->detail<QContactFamily>();
@@ -603,7 +692,7 @@ bool QVersitContactImporterPrivate::createFamily(
         return false;
     }
 
-    saveDetailWithContext(contact, &family, extractContexts(property));
+    saveDetailWithContext(updatedDetails, family, extractContexts(property));
     return true;
 }
 
@@ -611,8 +700,11 @@ bool QVersitContactImporterPrivate::createFamily(
  * Creates a simple name-value contact detail.
  */
 bool QVersitContactImporterPrivate::createNameValueDetail(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
+    Q_UNUSED(contact)
     QString value(property.value());
     if (value.isEmpty())
         return false;
@@ -624,7 +716,7 @@ bool QVersitContactImporterPrivate::createNameValueDetail(
     QContactDetail detail(nameAndValueType.first);
     detail.setValue(nameAndValueType.second, value);
 
-    saveDetailWithContext(contact, &detail, extractContexts(property));
+    saveDetailWithContext(updatedDetails, detail, extractContexts(property));
     return true;
 }
 
@@ -632,7 +724,9 @@ bool QVersitContactImporterPrivate::createNameValueDetail(
  * Find an existing QContactName and set the CustomLabel field on it
  */
 bool QVersitContactImporterPrivate::createCustomLabel(
-    const QVersitProperty& property, QContact* contact) const
+    const QVersitProperty& property,
+    QContact* contact,
+    QList<QContactDetail>* updatedDetails)
 {
     QString label(property.value());
     if (!label.isEmpty()) {
@@ -644,7 +738,7 @@ bool QVersitContactImporterPrivate::createCustomLabel(
 
         name.setCustomLabel(property.value());
 
-        saveDetailWithContext(contact, &name, extractContexts(property));
+        saveDetailWithContext(updatedDetails, name, extractContexts(property));
         return true;
     } else {
         return false;
@@ -748,15 +842,17 @@ QString QVersitContactImporterPrivate::saveContentToFile(
 }
 
 /*!
- * Saves \a detail to the \a contact.  Also sets the contexts to \a contexts if it is not empty.
+ * Adds \a detail to the \a updatedDetails list.  Also sets the contexts to \a contexts if it is not
+ * empty.
  */
 void QVersitContactImporterPrivate::saveDetailWithContext(
-        QContact* contact, QContactDetail* detail, const QStringList& contexts) const
+        QList<QContactDetail>* updatedDetails,
+        QContactDetail detail,
+        const QStringList& contexts)
 {
-    if (!contexts.isEmpty()) {
-        detail->setContexts(contexts);
-    }
-    contact->saveDetail(detail);
+    if (!contexts.isEmpty())
+        detail.setContexts(contexts);
+    updatedDetails->append(detail);
 }
 
 /*! Synthesize the display label from the name of the contact, or, failing that, the nickname of
