@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,17 +33,14 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
+
+//TESTED_COMPONENT=src/location
+
 #include "../qlocationtestutils_p.h"
 
 #include <qgeocoordinate.h>
@@ -334,6 +331,10 @@ private slots:
                 << QGeoCoordinate(-90,-180) << QGeoCoordinate(-45,+45) << false;
         QTest::newRow("valid == same valid")
                 << QGeoCoordinate(-90,-180) << QGeoCoordinate(-90,-180) << true;
+        QTest::newRow("different longitudes at north pole are equal")
+                << QGeoCoordinate(90, 0) << QGeoCoordinate(90, 45) << true;
+        QTest::newRow("different longitudes at south pole are equal")
+                << QGeoCoordinate(-90, 0) << QGeoCoordinate(-90, 45) << true;
     }
 
     void type()
