@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,28 +33,26 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+//TESTED_COMPONENT=src/multimedia
+
 #include <qmobilityglobal.h>
 #include <QtTest/QtTest>
 
 #include <QtCore/qdir.h>
 
+#include <qgraphicsvideoitem.h>
 #include <qmediaimageviewer.h>
 #include <qmediaimageviewerservice_p.h>
 #include <qmediaplaylist.h>
 #include <qmediaservice.h>
 #include <qvideorenderercontrol.h>
+#include <qvideowidget.h>
 #include <qvideowidgetcontrol.h>
 
 #include <QtCore/qfile.h>
@@ -88,6 +86,7 @@ private slots:
     void invalidPlaylist();
     void elapsedTime();
     void rendererControl();
+    void setVideoOutput();
 
 public:
     tst_QMediaImageViewer() : m_network(0), m_imageDir(QLatin1String(TESTDATA_DIR)) {}
@@ -981,6 +980,30 @@ void tst_QMediaImageViewer::rendererControl()
     QCOMPARE(rendererControl->surface(), nullSurface);
 }
 
+void tst_QMediaImageViewer::setVideoOutput()
+{
+    QMediaImageViewer imageViewer;
+
+    QVideoWidget widget;
+    QGraphicsVideoItem item;
+
+    imageViewer.setVideoOutput(&widget);
+    QVERIFY(widget.mediaObject() == &imageViewer);
+
+    imageViewer.setVideoOutput(&item);
+    QVERIFY(widget.mediaObject() == 0);
+    QVERIFY(item.mediaObject() == &imageViewer);
+
+    imageViewer.setVideoOutput(reinterpret_cast<QVideoWidget *>(0));
+    QVERIFY(item.mediaObject() == 0);
+
+    imageViewer.setVideoOutput(&widget);
+    QVERIFY(widget.mediaObject() == &imageViewer);
+
+    imageViewer.setVideoOutput(reinterpret_cast<QGraphicsVideoItem *>(0));
+    QVERIFY(widget.mediaObject() == 0);
+}
+
 QTEST_MAIN(tst_QMediaImageViewer)
 
 #include "tst_qmediaimageviewer.moc"
