@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,22 +33,18 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
+
+//TESTED_COMPONENT=src/messaging
+
 #include <QObject>
 #include <QTest>
-#include <QDebug>
 
-#include "qtmessaging.h"
+#include "qmessageservice.h"
 #include "../support/support.h"
 
 #if defined(Q_OS_SYMBIAN) || (defined(Q_OS_WIN) && defined(_WIN32_WCE))
@@ -467,7 +463,7 @@ void tst_QMessageService::testQueryMessages()
             QTRY_VERIFY(sc.state == QMessageService::FinishedState);
 #else
             while(testService->state() == QMessageService::ActiveState)
-				QTest::qWait(1000);
+				QTest::qWait(50);
 #endif
             QCOMPARE(sc.ids.toSet().subtract(existingMessageIds),ids.toSet());
 
@@ -478,7 +474,7 @@ void tst_QMessageService::testQueryMessages()
             QTRY_VERIFY(sc.state == QMessageService::FinishedState);
 #else
             while(testService->state() == QMessageService::ActiveState)
-                QTest::qWait(1000);
+                QTest::qWait(50);
 #endif
             QCOMPARE(sc.ids.toSet().subtract(existingMessageIds),negatedIds.toSet());
 
@@ -491,7 +487,7 @@ void tst_QMessageService::testQueryMessages()
             QTRY_VERIFY(sc.state == QMessageService::FinishedState);
 #else
             while(testService->state() == QMessageService::ActiveState)
-                QTest::qWait(1000);
+                QTest::qWait(50);
 #endif
             QCOMPARE(sc.ids.toSet().subtract(existingMessageIds),ids.toSet());
 
@@ -502,7 +498,7 @@ void tst_QMessageService::testQueryMessages()
             QTRY_VERIFY(sc.state == QMessageService::FinishedState);
 #else
             while(testService->state() == QMessageService::ActiveState)
-                QTest::qWait(1000);
+                QTest::qWait(50);
 #endif
             QCOMPARE(sc.ids.toSet().subtract(existingMessageIds),negatedIds.toSet());
         }
@@ -518,13 +514,11 @@ void tst_QMessageService::testQueryCountData()
     QTest::addColumn<QMessageIdList>("negatedIds");
     QTest::addColumn<QString>("body");
 
-#if !defined(Q_WS_MAEMO_5) && !defined(Q_WS_MAEMO_6)
     QTest::newRow("empty filter")
         << QMessageFilter() 
         << messageIds
         << QMessageIdList()
         << "";
-#endif
 
     QTest::newRow("id equality 1")
         << QMessageFilter::byId(messageIds[0], QMessageDataComparator::Equal) 
@@ -1949,7 +1943,7 @@ void tst_QMessageService::testCountMessages()
 
         SignalCatcher sc(this);
         connect(testService,SIGNAL(messagesCounted(int)),&sc,SLOT(messagesCounted(int)));
-#ifdef Q_OS_SYMBIAN
+#if (defined(Q_OS_SYMBIAN) || defined(Q_WS_MAEMO_5) || defined(Q_WS_MAEMO_6))
         connect(testService,SIGNAL(stateChanged(QMessageService::State)),&sc,SLOT(stateChanged(QMessageService::State)));
 #endif
 
@@ -1957,20 +1951,20 @@ void tst_QMessageService::testCountMessages()
 
         if(body.isEmpty()) {
             QCOMPARE(testService->countMessages(filter&~existingAccountsFilter),true);
-#ifdef Q_OS_SYMBIAN
+#if (defined(Q_OS_SYMBIAN) || defined(Q_WS_MAEMO_5) || defined(Q_WS_MAEMO_6))
             QTRY_VERIFY(sc.state == QMessageService::FinishedState);
 #else
             while(testService->state() == QMessageService::ActiveState)
-				QTest::qWait(1000);
+				QTest::qWait(50);
 #endif
             QCOMPARE(sc.count-existingMessageIds.count(), ids.count());
 
             QCOMPARE(testService->countMessages(~filter&~existingAccountsFilter),true);
-#ifdef Q_OS_SYMBIAN
+#if (defined(Q_OS_SYMBIAN) || defined(Q_WS_MAEMO_5) || defined(Q_WS_MAEMO_6))
             QTRY_VERIFY(sc.state == QMessageService::FinishedState);
 #else
             while(testService->state() == QMessageService::ActiveState)
-                QTest::qWait(1000);
+                QTest::qWait(50);
 #endif
             QCOMPARE(sc.count-existingMessageIds.count(), negatedIds.count());
         }
