@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,18 +33,14 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+//TESTED_COMPONENT=src/bearer
+
 #include <QtTest/QtTest>
 #include <QLocalServer>
 #include <QLocalSocket>
@@ -678,7 +674,7 @@ void tst_QNetworkSession::sessionStop()
         QVERIFY(openSession(&innocentSession));    
         qDebug("Waiting for %d ms after open to make sure all platform indications are propagated", configWaitdelayInMs);
         QTest::qWait(configWaitdelayInMs);
-    qDebug("----------4.2 Calling closedSession.stop()");
+        qDebug("----------4.2 Calling closedSession.stop()");
         closedSession.stop();
         qDebug("Waiting for %d ms to get all configurationChange signals from platform..", configWaitdelayInMs);
         QTest::qWait(configWaitdelayInMs); // Wait to get all relevant configurationChange() signals
@@ -884,7 +880,7 @@ void tst_QNetworkSession::sessionOpenCloseStop()
     // The remaining tests require the session to be not NotAvailable.
     if (session.state() == QNetworkSession::NotAvailable)
         QSKIP("Network is not available.", SkipSingle);
-
+        
     QSignalSpy sessionOpenedSpy(&session, SIGNAL(opened()));
     QSignalSpy sessionClosedSpy(&session, SIGNAL(closed()));
     QSignalSpy stateChangedSpy(&session, SIGNAL(stateChanged(QNetworkSession::State)));
@@ -912,9 +908,14 @@ void tst_QNetworkSession::sessionOpenCloseStop()
         session.waitForOpened();
 #endif        
 
+         // Wait until the configuration is uptodate as well, it may be signaled 'connected' 
+         // bit later than the session
+         QTRY_VERIFY(configuration.state() == QNetworkConfiguration::Active);
+
         if (session.isOpen())
             QVERIFY(!sessionOpenedSpy.isEmpty() || !errorSpy.isEmpty());
         if (!errorSpy.isEmpty()) {
+        
             QNetworkSession::SessionError error =
                 qvariant_cast<QNetworkSession::SessionError>(errorSpy.first().at(0));
 
@@ -1078,21 +1079,28 @@ void tst_QNetworkSession::sessionOpenCloseStop()
                 if (configuration.type() == QNetworkConfiguration::ServiceNetwork) {
                     bool roamedSuccessfully = false;
 
-                    QCOMPARE(stateChangedSpy2.count(), 4);
+                    QNetworkSession::State state;
+                    if (stateChangedSpy2.count() == 4) {
+                        state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(0).at(0));
+                        QVERIFY(state == QNetworkSession::Connecting);
 
-                    QNetworkSession::State state =
-                        qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(0).at(0));
-                    QVERIFY(state == QNetworkSession::Connecting);
+                        state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(1).at(0));
+                        QVERIFY(state == QNetworkSession::Connected);
 
-                    state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(1).at(0));
-                    QVERIFY(state == QNetworkSession::Connected);
+                        state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(2).at(0));
+                        QVERIFY(state == QNetworkSession::Closing);
 
-                    state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(2).at(0));
-                    QVERIFY(state == QNetworkSession::Closing);
+                        state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(3).at(0));
+                        QVERIFY(state == QNetworkSession::Disconnected);
+                    } else if (stateChangedSpy2.count() == 2) {
+                        state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(0).at(0));
+                        QVERIFY(state == QNetworkSession::Closing);
 
-                    state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(3).at(0));
-                    QVERIFY(state == QNetworkSession::Disconnected);
-                    
+                        state = qvariant_cast<QNetworkSession::State>(stateChangedSpy2.at(1).at(0));
+                        QVERIFY(state == QNetworkSession::Disconnected);
+                    } else {
+                        QFAIL("Unexpected amount of state changes when roaming.");
+                    }
 			
                     QTRY_VERIFY(session.state() == QNetworkSession::Roaming ||
                                 session.state() == QNetworkSession::Connected ||
@@ -1114,8 +1122,9 @@ void tst_QNetworkSession::sessionOpenCloseStop()
                         roamedSuccessfully = true;
                     } else if (state == QNetworkSession::Closing) {
                         QTRY_VERIFY(session2.state() == QNetworkSession::Disconnected);
-                        QTRY_VERIFY(session.state() == QNetworkSession::Connected);
-                        roamedSuccessfully = true;
+                        QTRY_VERIFY(session.state() == QNetworkSession::Connected || 
+                        	session.state() == QNetworkSession::Disconnected);
+                        roamedSuccessfully = false;
                     } else if (state == QNetworkSession::Disconnected) {
                         QTRY_VERIFY(!errorSpy.isEmpty());
                         QTRY_VERIFY(session2.state() == QNetworkSession::Disconnected);
@@ -1127,16 +1136,18 @@ void tst_QNetworkSession::sessionOpenCloseStop()
                             QVERIFY(state == QNetworkSession::Roaming);
                         }
                         roamedSuccessfully = true;
-                  	}
+                    }
 
                     if (roamedSuccessfully) {
+                        // Verify that you can open session based on the disconnected configuration
                         QString configId = session.sessionProperty("ActiveConfiguration").toString();
-                        QNetworkConfiguration config = manager.configurationFromIdentifier(configId); 
+                        QNetworkConfiguration config = manager.configurationFromIdentifier(configId);
                         QNetworkSession session3(config);
                         QSignalSpy errorSpy3(&session3, SIGNAL(error(QNetworkSession::SessionError)));
                         QSignalSpy sessionOpenedSpy3(&session3, SIGNAL(opened()));
                         session3.open();
-                        session3.waitForOpened();      
+                        session3.waitForOpened();
+                        QTest::qWait(1000); // Wait awhile to get all signals from platform
                         if (session.isOpen())
                             QVERIFY(!sessionOpenedSpy3.isEmpty() || !errorSpy3.isEmpty());
                         session.stop();
