@@ -8,9 +8,9 @@
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Solutions Commercial License Agreement provided
-** with the Software or, alternatively, in accordance with the terms
+** Licensees holding valid Qt Commercial licenses may use this file in 
+** accordance with the Qt Commercial License Agreement provided with
+** the Software or, alternatively, in accordance with the terms
 ** contained in a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
@@ -33,18 +33,14 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** Please note Third Party Software included with Qt Solutions may impose
-** additional restrictions and it is the user's responsibility to ensure
-** that they have met the licensing requirements of the GPL, LGPL, or Qt
-** Solutions Commercial license and the relevant license of the Third
-** Party Software they are using.
-**
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+//TESTED_COMPONENT=src/publishsubscribe
+
 #include "tst_qvaluespacesubscribershared.h"
 
 #include <qvaluespacesubscriber.h>
@@ -551,6 +547,23 @@ void tst_QValueSpaceSubscriber::testPathChanges()
     QVERIFY(subscriber.subPaths().toSet().contains(homeUserPaths.toSet()));
 }
 
+void tst_QValueSpaceSubscriber::testInvalidPath_data()
+{
+    QTest::addColumn<QString>("subscriber_path");
+    QTest::newRow("One char missing") << "/hom";
+    QTest::newRow("Two chars missing") << "/ho";
+    QTest::newRow("One char missing in subdir") << "/home/use";
+    QTest::newRow("Two chars missing in subdir") << "/home/us";
+}
+
+void tst_QValueSpaceSubscriber::testInvalidPath()
+{
+    QFETCH(QString, subscriber_path);
+
+    QValueSpaceSubscriber subscriber(subscriber_path);
+    QVERIFY(subscriber.subPaths().isEmpty());
+}
+
 void tst_QValueSpaceSubscriber::contentsChanged_data()
 {
     QTest::addColumn<QAbstractValueSpaceLayer *>("layer");
